from _typeshed import Incomplete

from networkx.utils.backends import _dispatchable

from ..classes.graph import Graph

__all__ = [
    "fast_gnp_random_graph",
    "gnp_random_graph",
    "dense_gnm_random_graph",
    "gnm_random_graph",
    "erdos_renyi_graph",
    "binomial_graph",
    "newman_watts_strogatz_graph",
    "watts_strogatz_graph",
    "connected_watts_strogatz_graph",
    "random_regular_graph",
    "barabasi_albert_graph",
    "dual_barabasi_albert_graph",
    "extended_barabasi_albert_graph",
    "powerlaw_cluster_graph",
    "random_lobster",
    "random_shell_graph",
    "random_powerlaw_tree",
    "random_powerlaw_tree_sequence",
    "random_kernel_graph",
]

@_dispatchable
def fast_gnp_random_graph(n, p, seed=None, directed: bool = False): ...
@_dispatchable
def gnp_random_graph(n, p, seed=None, directed: bool = False): ...

binomial_graph = gnp_random_graph
erdos_renyi_graph = gnp_random_graph

@_dispatchable
def dense_gnm_random_graph(n, m, seed=None): ...
@_dispatchable
def gnm_random_graph(n, m, seed=None, directed: bool = False): ...
@_dispatchable
def newman_watts_strogatz_graph(n, k, p, seed=None): ...
@_dispatchable
def watts_strogatz_graph(n, k, p, seed=None): ...
@_dispatchable
def connected_watts_strogatz_graph(n, k, p, tries: int = 100, seed=None): ...
@_dispatchable
def random_regular_graph(d, n, seed=None): ...
@_dispatchable
def barabasi_albert_graph(n, m, seed=None, initial_graph=None) -> Graph[Incomplete]: ...
@_dispatchable
def dual_barabasi_albert_graph(n, m1, m2, p, seed=None, initial_graph=None) -> Graph[Incomplete]: ...
@_dispatchable
def extended_barabasi_albert_graph(n, m, p, q, seed=None) -> Graph[Incomplete]: ...
@_dispatchable
def powerlaw_cluster_graph(n, m, p, seed=None): ...
@_dispatchable
def random_lobster(n, p1, p2, seed=None): ...
@_dispatchable
def random_shell_graph(constructor, seed=None): ...
@_dispatchable
def random_powerlaw_tree(n, gamma: float = 3, seed=None, tries: int = 100): ...
@_dispatchable
def random_powerlaw_tree_sequence(n, gamma: float = 3, seed=None, tries: int = 100): ...
@_dispatchable
def random_kernel_graph(n, kernel_integral, kernel_root=None, seed=None): ...



# Commonly used type aliases.
# Everything in this module is private for stubs. There is no runtime equivalent.

from collections.abc import Iterable, Mapping, Sequence
from typing import Any, Protocol, TypeVar
from typing_extensions import TypeAlias

import numpy as np
import numpy.typing as npt
import tensorflow as tf
from tensorflow.dtypes import DType
from tensorflow.keras.layers import InputSpec

_T = TypeVar("_T")
ContainerGeneric: TypeAlias = Mapping[str, ContainerGeneric[_T]] | Sequence[ContainerGeneric[_T]] | _T

TensorLike: TypeAlias = tf.Tensor | tf.RaggedTensor | tf.SparseTensor
SparseTensorLike: TypeAlias = tf.Tensor | tf.SparseTensor
RaggedTensorLike: TypeAlias = tf.Tensor | tf.RaggedTensor
# _RaggedTensorLikeT = TypeVar("_RaggedTensorLikeT", tf.Tensor, tf.RaggedTensor)
Gradients: TypeAlias = tf.Tensor | tf.IndexedSlices

class KerasSerializable1(Protocol):
    def get_config(self) -> dict[str, Any]: ...

class KerasSerializable2(Protocol):
    __name__: str

KerasSerializable: TypeAlias = KerasSerializable1 | KerasSerializable2

TensorValue: TypeAlias = tf.Tensor  # Alias for a 0D Tensor
Integer: TypeAlias = TensorValue | int | IntArray | np.number[Any]  # Here IntArray are assumed to be 0D.
Float: TypeAlias = Integer | float | FloatArray
Slice: TypeAlias = tf.Tensor | tf.RaggedTensor | int | slice | None
FloatDataSequence: TypeAlias = Sequence[float] | Sequence[FloatDataSequence]
IntDataSequence: TypeAlias = Sequence[int] | Sequence[IntDataSequence]
StrDataSequence: TypeAlias = Sequence[str] | Sequence[StrDataSequence]
DataSequence: TypeAlias = FloatDataSequence | StrDataSequence | IntDataSequence
ScalarTensorCompatible: TypeAlias = tf.Tensor | str | float | np.ndarray[Any, Any] | np.number[Any]
UIntTensorCompatible: TypeAlias = tf.Tensor | int | UIntArray
FloatTensorCompatible: TypeAlias = tf.Tensor | int | IntArray | float | FloatArray | np.number[Any]
StringTensorCompatible: TypeAlias = tf.Tensor | str | npt.NDArray[np.str_] | Sequence[StringTensorCompatible]

TensorCompatible: TypeAlias = ScalarTensorCompatible | Sequence[TensorCompatible]
# _TensorCompatibleT = TypeVar("_TensorCompatibleT", bound=TensorCompatible)
# Sparse tensors are very annoying. Some operations work on them, but many do not.
# You will need to manually verify if an operation supports them. SparseTensorCompatible is intended to be a
# broader type than TensorCompatible and not all operations will support broader version. If unsure,
# use TensorCompatible instead.
SparseTensorCompatible: TypeAlias = TensorCompatible | tf.SparseTensor
# TensorFlow tries to convert anything passed as input. Meaning that even if, for example, only a Tensor of int32
# is allowed, a numpy array of strings that can be converted to int32 will work. Therefore having anything more specific
# then AnyArray might cause false positives, while AnyArray might cause false negatives.
TensorOrArray: TypeAlias = tf.Tensor | AnyArray

ShapeLike: TypeAlias = tf.TensorShape | Iterable[ScalarTensorCompatible | None] | int | tf.Tensor
DTypeLike: TypeAlias = DType | str | np.dtype[Any] | int

ContainerTensors: TypeAlias = ContainerGeneric[tf.Tensor]
ContainerTensorsLike: TypeAlias = ContainerGeneric[TensorLike]
ContainerTensorCompatible: TypeAlias = ContainerGeneric[TensorCompatible]
ContainerGradients: TypeAlias = ContainerGeneric[Gradients]
ContainerTensorShape: TypeAlias = ContainerGeneric[tf.TensorShape]
ContainerInputSpec: TypeAlias = ContainerGeneric[InputSpec]

AnyArray: TypeAlias = npt.NDArray[Any]
FloatArray: TypeAlias = npt.NDArray[np.float16 | np.float32 | np.float64]
UIntArray: TypeAlias = npt.NDArray[np.uint | np.uint8 | np.uint16 | np.uint32 | np.uint64]
SignedIntArray: TypeAlias = npt.NDArray[np.int_ | np.int8 | np.int16 | np.int32 | np.int64]
IntArray: TypeAlias = UIntArray | SignedIntArray



from _typeshed import Incomplete

from ...extend.operation import ExtendedOperation
from ...protocol.novell import EndGroupTypeRequestValue, EndGroupTypeResponseValue

class EndTransaction(ExtendedOperation):
    request_name: str
    response_name: str
    request_value: EndGroupTypeRequestValue
    asn1_spec: EndGroupTypeResponseValue
    def config(self) -> None: ...
    def __init__(self, connection, commit: bool = True, controls=None) -> None: ...
    def populate_result(self) -> None: ...
    response_value: Incomplete
    def set_response(self) -> None: ...



from typing import TypedDict, type_check_only
from typing_extensions import Self

@type_check_only
class _ParamsDict(TypedDict, total=False):
    businessName: str
    returnUrl: str
    cancelUrl: str

class BankAccountInstantVerificationJwtRequest:
    def __init__(self) -> None: ...
    def business_name(self, business_name: str) -> Self: ...
    def return_url(self, return_url: str) -> Self: ...
    def cancel_url(self, cancel_url: str) -> Self: ...
    def get_business_name(self) -> str: ...
    def get_return_url(self) -> str: ...
    def get_cancel_url(self) -> str: ...
    def to_graphql_variables(self) -> _ParamsDict: ...



from typing import overload

import numpy as np
from numpy.typing import NDArray

from ._typing import ArrayLike, ArrayLikeSeq, GeoArray, OptGeoArrayLike, OptGeoArrayLikeSeq
from .geometry import GeometryCollection, LineString, MultiLineString, Point
from .lib import Geometry

__all__ = ["line_interpolate_point", "line_locate_point", "line_merge", "shared_paths", "shortest_line"]

@overload
def line_interpolate_point(line: None, distance: float, normalized: bool = False, **kwargs) -> None: ...
@overload
def line_interpolate_point(
    line: None, distance: ArrayLikeSeq[float], normalized: bool = False, **kwargs
) -> NDArray[np.object_]: ...  # Array of None
@overload
def line_interpolate_point(
    line: LineString | MultiLineString | GeometryCollection, distance: float, normalized: bool = False, **kwargs
) -> Point: ...
@overload
def line_interpolate_point(
    line: LineString | MultiLineString | GeometryCollection, distance: ArrayLikeSeq[float], normalized: bool = False, **kwargs
) -> GeoArray: ...
@overload
def line_interpolate_point(
    line: OptGeoArrayLikeSeq, distance: ArrayLike[float], normalized: bool = False, **kwargs
) -> GeoArray: ...
@overload
def line_locate_point(
    line: LineString | MultiLineString | GeometryCollection | None, other: Point | None, normalized: bool = False, **kwargs
) -> np.float64: ...
@overload
def line_locate_point(
    line: LineString | MultiLineString | GeometryCollection | None, other: OptGeoArrayLikeSeq, normalized: bool = False, **kwargs
) -> NDArray[np.float64]: ...
@overload
def line_locate_point(
    line: OptGeoArrayLikeSeq, other: OptGeoArrayLike, normalized: bool = False, **kwargs
) -> NDArray[np.float64]: ...
@overload
def line_merge(line: None, directed: bool = False, **kwargs) -> None: ...
@overload
def line_merge(line: Geometry, directed: bool = False, **kwargs) -> LineString | MultiLineString | GeometryCollection: ...
@overload
def line_merge(line: OptGeoArrayLikeSeq, directed: bool = False, **kwargs) -> GeoArray: ...
@overload
def shared_paths(a: LineString | MultiLineString | None, b: None, **kwargs) -> None: ...
@overload
def shared_paths(a: None, b: LineString | MultiLineString | None, **kwargs) -> None: ...
@overload
def shared_paths(
    a: LineString | MultiLineString, b: LineString | MultiLineString, **kwargs
) -> GeometryCollection[MultiLineString]: ...
@overload
def shared_paths(a: LineString | MultiLineString | None, b: OptGeoArrayLikeSeq, **kwargs) -> GeoArray: ...
@overload
def shared_paths(a: OptGeoArrayLikeSeq, b: OptGeoArrayLike, **kwargs) -> GeoArray: ...
@overload
def shortest_line(a: Geometry | None, b: None, **kwargs) -> None: ...
@overload
def shortest_line(a: None, b: Geometry | None, **kwargs) -> None: ...
@overload
def shortest_line(a: Geometry, b: Geometry, **kwargs) -> LineString: ...
@overload
def shortest_line(a: Geometry | None, b: OptGeoArrayLikeSeq, **kwargs) -> GeoArray: ...
@overload
def shortest_line(a: OptGeoArrayLikeSeq, b: OptGeoArrayLike, **kwargs) -> GeoArray: ...



from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["kl_connected_subgraph", "is_kl_connected"]

@_dispatchable
def kl_connected_subgraph(G: Graph[_Node], k: int, l: int, low_memory: bool = False, same_as_graph: bool = False): ...
@_dispatchable
def is_kl_connected(G: Graph[_Node], k: int, l: int, low_memory: bool = False) -> bool: ...



from hvac.api.vault_api_base import VaultApiBase

DEFAULT_MOUNT_POINT: str

class Consul(VaultApiBase):
    def configure_access(self, address, token, scheme=None, mount_point="consul"): ...
    def create_or_update_role(
        self, name, policy=None, policies=None, token_type=None, local=None, ttl=None, max_ttl=None, mount_point="consul"
    ): ...
    def read_role(self, name, mount_point="consul"): ...
    def list_roles(self, mount_point="consul"): ...
    def delete_role(self, name, mount_point="consul"): ...
    def generate_credentials(self, name, mount_point="consul"): ...



from _typeshed import Incomplete
from collections.abc import Mapping, Sequence

from networkx.utils.backends import _dispatchable
from numpy.random import RandomState

from ..classes.graph import Graph

__all__ = ["is_valid_joint_degree", "is_valid_directed_joint_degree", "joint_degree_graph", "directed_joint_degree_graph"]

@_dispatchable
def is_valid_joint_degree(joint_degrees: Mapping[int, Mapping[int, int]]) -> bool: ...
@_dispatchable
def joint_degree_graph(
    joint_degrees: Mapping[int, Mapping[int, int]], seed: int | RandomState | None = None
) -> Graph[Incomplete]: ...
@_dispatchable
def is_valid_directed_joint_degree(
    in_degrees: Sequence[int], out_degrees: Sequence[int], nkk: Mapping[int, Mapping[int, int]]
) -> bool: ...
@_dispatchable
def directed_joint_degree_graph(
    in_degrees: Sequence[int],
    out_degrees: Sequence[int],
    nkk: Mapping[int, Mapping[int, int]],
    seed: int | RandomState | None = None,
) -> Graph[Incomplete]: ...



from _typeshed import Incomplete

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable
from numpy.random import RandomState

__all__ = ["rich_club_coefficient"]

@_dispatchable
def rich_club_coefficient(
    G: Graph[_Node], normalized: bool = True, Q: float = 100, seed: int | RandomState | None = None
) -> dict[Incomplete, Incomplete]: ...



import datetime
from typing_extensions import TypeAlias

from .cal import Alarm, Event, Todo
from .error import (
    ComponentEndMissing as ComponentEndMissing,
    ComponentStartMissing as ComponentStartMissing,
    IncompleteAlarmInformation as IncompleteAlarmInformation,
    LocalTimezoneMissing as LocalTimezoneMissing,
)

__all__ = ["Alarms", "AlarmTime", "IncompleteAlarmInformation", "ComponentEndMissing", "ComponentStartMissing"]

Parent: TypeAlias = Event | Todo

class AlarmTime:
    def __init__(
        self,
        alarm: Alarm,
        trigger: datetime.datetime,
        acknowledged_until: datetime.datetime | None = None,
        snoozed_until: datetime.datetime | None = None,
        parent: Parent | None = None,
    ): ...
    @property
    def acknowledged(self) -> datetime.datetime | None: ...
    @property
    def alarm(self) -> Alarm: ...
    @property
    def parent(self) -> Parent | None: ...
    def is_active(self) -> bool: ...
    @property
    def trigger(self) -> datetime.date: ...

class Alarms:
    def __init__(self, component: Alarm | Event | Todo | None = None): ...
    def add_component(self, component: Alarm | Parent) -> None: ...
    def set_parent(self, parent: Parent) -> None: ...
    def add_alarm(self, alarm: Alarm) -> None: ...
    def set_start(self, dt: datetime.date | None) -> None: ...
    def set_end(self, dt: datetime.date | None) -> None: ...
    def acknowledge_until(self, dt: datetime.date | None) -> None: ...
    def snooze_until(self, dt: datetime.date | None) -> None: ...
    def set_local_timezone(self, tzinfo: datetime.tzinfo | str | None) -> None: ...
    @property
    def times(self) -> list[AlarmTime]: ...
    @property
    def active(self) -> list[AlarmTime]: ...



from _typeshed import Incomplete

from braintree.braintree_gateway import BraintreeGateway

class ClientToken:
    @staticmethod
    def generate(params: dict[str, Incomplete] | None = None, gateway: BraintreeGateway | None = None) -> str: ...
    @staticmethod
    def generate_signature() -> list[str | dict[str, list[str]]]: ...



mathaccent: dict[str, str]
mathalpha: dict[str, str]
mathbin: dict[str, str]
mathclose: dict[str, str]
mathfence: dict[str, str]
mathop: dict[str, str]
mathopen: dict[str, str]
mathord: dict[str, str]
mathover: dict[str, str]
mathpunct: dict[str, str]
mathradical: dict[str, str]
mathrel: dict[str, str]
mathunder: dict[str, str]
space: dict[str, str]



from gdb import Progspace
from gdb.missing_files import MissingFileHandler

class MissingObjfileHandler(MissingFileHandler):
    def __call__(self, buildid: str, filename: str) -> bool | str | None: ...

def register_handler(locus: Progspace | None, handler: MissingObjfileHandler, replace: bool = False) -> None: ...



import types
from _typeshed import FileDescriptorLike, FileDescriptorOrPath, HasFileno, StrOrBytesPath
from inspect import _IntrospectableCallable, _ParameterKind
from socket import socket
from typing import Any, Literal, NoReturn
from urllib.parse import SplitResult

from ._types import _AddressType, _WSGIAppType

REDIRECT_TO: str
hop_headers: set[str]

def load_entry_point(distribution: str, group: str, name: str) -> type[object]: ...
def load_class(
    uri: str | object, default: str = "gunicorn.workers.sync.SyncWorker", section: str = "gunicorn.workers"
) -> type[Any]: ...

positionals: tuple[Literal[_ParameterKind.POSITIONAL_ONLY], Literal[_ParameterKind.POSITIONAL_OR_KEYWORD]]

def get_arity(f: _IntrospectableCallable) -> int: ...
def get_username(uid: int) -> str: ...
def set_owner_process(uid: int, gid: int, initgroups: bool = False) -> None: ...
def chown(path: FileDescriptorOrPath, uid: int, gid: int) -> None: ...
def unlink(filename: StrOrBytesPath) -> None: ...
def is_ipv6(addr: str) -> bool: ...
def parse_address(netloc: str, default_port: str = "8000") -> _AddressType: ...
def close_on_exec(fd: FileDescriptorLike) -> None: ...
def set_non_blocking(fd: FileDescriptorLike) -> None: ...
def close(sock: socket) -> None: ...
def write_chunk(sock: socket, data: bytes) -> None: ...
def write(sock: socket, data: bytes, chunked: bool = False) -> None: ...
def write_nonblock(sock: socket, data: bytes, chunked: bool = False) -> None: ...
def write_error(sock: socket, status_int: int, reason: str, mesg: str) -> None: ...
def import_app(module: str) -> _WSGIAppType: ...
def getcwd() -> str: ...
def http_date(timestamp: float | None = None) -> str: ...
def is_hoppish(header: str) -> bool: ...
def daemonize(enable_stdio_inheritance: bool = False) -> None: ...
def seed() -> None: ...
def check_is_writable(path: FileDescriptorOrPath) -> None: ...
def to_bytestring(value: str | bytes, encoding: str = "utf8") -> bytes: ...
def has_fileno(obj: HasFileno) -> bool: ...
def warn(msg: str) -> None: ...
def make_fail_app(msg: str) -> _WSGIAppType: ...
def split_request_uri(uri: str) -> SplitResult: ...
def reraise(tp: type[BaseException], value: BaseException | None, tb: types.TracebackType | None = None) -> NoReturn: ...
def bytes_to_str(b: bytes) -> str: ...
def unquote_to_wsgi_str(string: str) -> str: ...



from _typeshed import Incomplete
from logging import Logger

from oauthlib.common import Request
from oauthlib.oauth2.rfc6749.request_validator import RequestValidator as OAuth2RequestValidator

log: Logger

class Dispatcher:
    default_grant: Incomplete | None
    oidc_grant: Incomplete | None

class AuthorizationCodeGrantDispatcher(Dispatcher):
    default_grant: Incomplete | None
    oidc_grant: Incomplete | None
    def __init__(self, default_grant=None, oidc_grant=None) -> None: ...
    def create_authorization_response(self, request: Request, token_handler): ...
    def validate_authorization_request(self, request: Request): ...

class ImplicitTokenGrantDispatcher(Dispatcher):
    default_grant: Incomplete | None
    oidc_grant: Incomplete | None
    def __init__(self, default_grant=None, oidc_grant=None) -> None: ...
    def create_authorization_response(self, request: Request, token_handler): ...
    def validate_authorization_request(self, request: Request): ...

class AuthorizationTokenGrantDispatcher(Dispatcher):
    default_grant: Incomplete | None
    oidc_grant: Incomplete | None
    request_validator: OAuth2RequestValidator
    def __init__(self, request_validator: OAuth2RequestValidator, default_grant=None, oidc_grant=None) -> None: ...
    def create_token_response(self, request: Request, token_handler): ...



from typing import Any, Final
from typing_extensions import Self

__version__: Final[str]

class ABag:
    def __init__(self, **attr: Any) -> None: ...
    def clone(self, **attr: Any) -> Self: ...
    # ABag can have arbitrary attributes
    def __getattr__(self, name: str) -> Any: ...
    def __setattr__(self, name: str, value: Any) -> None: ...



import sys
from _typeshed import SupportsWrite as SupportsWrite
from collections.abc import Sequence
from typing import Any, Protocol, TypeVar, type_check_only
from typing_extensions import TypeAlias

import numpy as np
from numpy.typing import NDArray

from .lib import Geometry

if sys.version_info >= (3, 12):
    from collections.abc import Buffer

_T = TypeVar("_T")
_T_co = TypeVar("_T_co", covariant=True)
_DType = TypeVar("_DType", bound=np.dtype[Any])
_DType_co = TypeVar("_DType_co", covariant=True, bound=np.dtype[Any])

GeoT = TypeVar("GeoT", bound=Geometry)  # noqa: Y001
OptGeoT = TypeVar("OptGeoT", bound=Geometry | None)  # noqa: Y001

@type_check_only
class SupportsArray(Protocol[_DType_co]):
    def __array__(self) -> np.ndarray[Any, _DType_co]: ...

# TODO: revisit when mypy is happy with generic recursive type alias
# NestedSequence: TypeAlias = Sequence[_T] | Sequence[NestedSequence[_T]]
NestedSequence: TypeAlias = Sequence[_T] | Sequence[Sequence[_T]] | Sequence[Sequence[Sequence[_T]]]
DualArrayLike: TypeAlias = SupportsArray[_DType] | NestedSequence[SupportsArray[_DType]] | NestedSequence[_T]

# array-like sequences: objects accepted by np.array that produce at least 1-D arrays
if sys.version_info >= (3, 12):
    ArrayLikeSeq: TypeAlias = Buffer | DualArrayLike[np.dtype[Any], _T]
else:
    ArrayLikeSeq: TypeAlias = DualArrayLike[np.dtype[Any], _T]
GeoArrayLikeSeq: TypeAlias = ArrayLikeSeq[Geometry]
OptGeoArrayLikeSeq: TypeAlias = ArrayLikeSeq[Geometry | None]

# array-like: objects accepted by np.array that may also produce 0-D array
ArrayLike: TypeAlias = _T | ArrayLikeSeq[_T]
GeoArrayLike: TypeAlias = ArrayLike[Geometry]
OptGeoArrayLike: TypeAlias = ArrayLike[Geometry | None]

# There is no way to pronounce "array of BaseGeometry" currently because of the restriction on
# NDArray type variable to np.dtype and because np.object_ is not generic.
# Note the use of `BaseGeometry` instead of `Geometry` as the alias is used in return types.
GeoArray: TypeAlias = NDArray[np.object_]

@type_check_only
class SupportsGeoInterface(Protocol):
    @property
    def __geo_interface__(self) -> dict[str, Any]: ...

# Unlike _typeshed.SupportsRead, this protocol does not require a length parameter
@type_check_only
class SupportsRead(Protocol[_T_co]):
    def read(self) -> _T_co: ...



import re
from typing import Final, overload

from aws_xray_sdk.core.models.trace_header import TraceHeader

first_cap_re: Final[re.Pattern[str]]
all_cap_re: Final[re.Pattern[str]]
UNKNOWN_HOSTNAME: Final = "UNKNOWN HOST"

def inject_trace_header(headers, entity) -> None: ...
def calculate_sampling_decision(trace_header, recorder, sampling_req): ...
def construct_xray_header(headers) -> TraceHeader: ...
def calculate_segment_name(host_name, recorder): ...
def prepare_response_header(origin_header, segment) -> str: ...
def to_snake_case(name: str) -> str: ...
def strip_url(url): ...
@overload
def get_hostname(url: str | None) -> str: ...
@overload
def get_hostname(url: bytes | bytearray | None) -> str | bytes: ...
def unwrap(obj: object, attr: str) -> None: ...



from .base import BaseEndpoint as BaseEndpoint

class AuthorizationEndpoint(BaseEndpoint):
    def create_verifier(self, request, credentials): ...
    def create_authorization_response(
        self, uri, http_method: str = "GET", body=None, headers=None, realms=None, credentials=None
    ): ...
    def get_realms_and_credentials(self, uri, http_method: str = "GET", body=None, headers=None): ...



import sqlite3
from _typeshed import Incomplete, SupportsLenAndGetItem
from collections.abc import Container, Iterator, Mapping
from contextlib import AbstractContextManager
from typing import Any, Protocol, overload, type_check_only
from typing_extensions import TypeAlias

from pandas._typing import Scalar

from ..base import _ConvertibleToCRS
from ..geodataframe import GeoDataFrame

# Start SQLAlchemy hack
# ---------------------
# The code actually explicitly checks for SQLAlchemy's `Connection` and `Engine` with
# isinstance checks. However to avoid a dependency on SQLAlchemy, we use "good-enough"
# protocols that match as much as possible the SQLAlchemy implementation. This makes it
# very hard for someone to pass in the wrong object.
@type_check_only
class _SqlalchemyTransactionLike(Protocol):
    # is_active: bool
    # connection: _SqlalchemyConnectionLike
    # def __init__(self, connection: _SqlalchemyConnectionLike): ...
    # @property
    # def is_valid(self) -> bool: ...
    def close(self) -> None: ...
    def rollback(self) -> None: ...
    def commit(self) -> None: ...

# `Any` is used in places where it would require to copy a lot of types from sqlalchemy
@type_check_only
class _SqlAlchemyEventTarget(Protocol):
    dispatch: Any

@type_check_only
class _SqlalchemyConnectionLike(_SqlAlchemyEventTarget, Protocol):
    engine: Any
    @property
    def closed(self) -> bool: ...
    @property
    def invalidated(self) -> bool: ...
    def __enter__(self) -> _SqlalchemyConnectionLike: ...  # noqa: Y034
    def __exit__(self, type_, value, traceback, /) -> None: ...
    @property
    def info(self) -> dict[Any, Any]: ...
    def invalidate(self, exception: BaseException | None = None) -> None: ...
    def detach(self) -> None: ...
    def begin(self) -> _SqlalchemyTransactionLike: ...
    def commit(self) -> None: ...
    def rollback(self) -> None: ...
    def recover_twophase(self) -> list[Any]: ...
    def rollback_prepared(self, xid: Any, recover: bool = ...) -> None: ...
    def commit_prepared(self, xid: Any, recover: bool = ...) -> None: ...
    def in_transaction(self) -> bool: ...
    def in_nested_transaction(self) -> bool: ...
    def close(self) -> None: ...

@type_check_only
class _SqlalchemyEngineLike(_SqlAlchemyEventTarget, Protocol):
    dialect: Any
    pool: Any
    url: Any
    hide_parameters: bool
    @property
    def engine(self) -> _SqlalchemyEngineLike: ...
    def clear_compiled_cache(self) -> None: ...
    def update_execution_options(self, **opt: Any) -> None: ...
    @property
    def name(self) -> str: ...
    @property
    def driver(self) -> str: ...
    def dispose(self, close: bool = True) -> None: ...
    def begin(self) -> AbstractContextManager[Any]: ...
    def connect(self) -> Any: ...

_SqlalchemyConnectableLike: TypeAlias = _SqlalchemyConnectionLike | _SqlalchemyEngineLike
# ---------------------
# End SQLAlchemy hack

_SQLConnection: TypeAlias = str | _SqlalchemyConnectableLike | sqlite3.Connection  # coppied from pandas.io.sql

@overload
def _read_postgis(
    sql: str,
    con: _SQLConnection,
    geom_col: str = "geom",
    crs: _ConvertibleToCRS | None = None,
    index_col: str | Container[str] | None = None,
    coerce_float: bool = True,
    parse_dates: Container[str | Mapping[str, Incomplete]] | Mapping[str, str | Mapping[str, Incomplete]] | None = None,
    params: SupportsLenAndGetItem[Scalar] | Mapping[str, Scalar] | None = None,
    *,
    chunksize: int,
) -> Iterator[GeoDataFrame]: ...
@overload
def _read_postgis(
    sql: str,
    con: _SQLConnection,
    geom_col: str = "geom",
    crs: _ConvertibleToCRS | None = None,
    index_col: str | Container[str] | None = None,
    coerce_float: bool = True,
    parse_dates: Container[str | Mapping[str, Incomplete]] | Mapping[str, str | Mapping[str, Incomplete]] | None = None,
    params: SupportsLenAndGetItem[Scalar] | Mapping[str, Scalar] | None = None,
    chunksize: None = None,
) -> GeoDataFrame: ...
@overload
def _read_postgis(
    sql: str,
    con: _SQLConnection,
    geom_col: str = "geom",
    crs: _ConvertibleToCRS | None = None,
    index_col: str | Container[str] | None = None,
    coerce_float: bool = True,
    parse_dates: Container[str | Mapping[str, Incomplete]] | Mapping[str, str | Mapping[str, Incomplete]] | None = None,
    params: SupportsLenAndGetItem[Scalar] | Mapping[str, Scalar] | None = None,
    chunksize: int | None = None,
) -> GeoDataFrame | Iterator[GeoDataFrame]: ...



from _typeshed import Unused
from collections.abc import Mapping, Sequence

def launch(
    *,
    program: str | None = None,
    cwd: str | None = None,
    args: Sequence[str] = (),
    env: Mapping[str, str] | None = None,
    stopAtBeginningOfMainSubprogram: bool = False,
    **extra: Unused,
): ...
def attach(*, program: str | None = None, pid: int | None = None, target: str | None = None, **args: Unused) -> None: ...
def config_done(**args: Unused) -> None: ...



from __future__ import annotations

from collections.abc import Hashable
from typing import Any
from typing_extensions import assert_type

from cachetools import LRUCache, cached, keys as cachekeys
from cachetools.func import fifo_cache, lfu_cache, lru_cache, rr_cache, ttl_cache

# Tests for cachetools.cached

# Explicitly parameterize the cache to avoid Unknown types
cache_inst: LRUCache[int, int] = LRUCache(maxsize=128)


@cached(cache_inst)
def check_cached(x: int) -> int:
    return x * 2


assert_type(check_cached(3), int)
# Methods cache_info/cache_clear are only present when info=True; do not access them here.


@cached(cache_inst, info=True)
def check_cached_with_info(x: int) -> int:
    return x + 1


assert_type(check_cached_with_info(4), int)
assert_type(check_cached_with_info.cache_info().misses, int)
check_cached_with_info.cache_clear()


# Tests for cachetools.func decorators


@lru_cache
def lru_noparens(x: int) -> int:
    return x * 2


@lru_cache(maxsize=32)
def lru_with_maxsize(x: int) -> int:
    return x * 3


assert_type(lru_noparens(3), int)
assert_type(lru_with_maxsize(3), int)
assert_type(lru_noparens.cache_info().hits, int)
assert_type(lru_with_maxsize.cache_info().misses, int)
assert_type(lru_with_maxsize.cache_parameters(), dict[str, Any])
lru_with_maxsize.cache_clear()


@fifo_cache
def fifo_func(x: int) -> int:
    return x


@lfu_cache
def lfu_func(x: int) -> int:
    return x


@rr_cache
def rr_func(x: int) -> int:
    return x


@ttl_cache
def ttl_func(x: int) -> int:
    return x


assert_type(fifo_func(1), int)
assert_type(lfu_func(1), int)
assert_type(rr_func(1), int)
assert_type(ttl_func(1), int)
assert_type(fifo_func.cache_info().currsize, int)
assert_type(lfu_func.cache_parameters(), dict[str, Any])


# Tests for cachetools.keys

k1 = cachekeys.hashkey(1, "a")
assert_type(k1, tuple[Hashable, ...])


class C:
    def method(self, a: int) -> int:
        return a


inst = C()

k2 = cachekeys.methodkey(inst, 5)
assert_type(k2, tuple[Hashable, ...])

k3 = cachekeys.typedkey(1, "x")
assert_type(k3, tuple[Hashable, ...])

k4 = cachekeys.typedmethodkey(inst, 2)
assert_type(k4, tuple[Hashable, ...])



from typing import Any, SupportsIndex

from dateparser.calendars import non_gregorian_parser

class hijri:
    @classmethod
    def to_gregorian(cls, year: int | None = None, month: int | None = None, day: int | None = None) -> tuple[int, int, int]: ...
    @classmethod
    def from_gregorian(
        cls, year: SupportsIndex | None = None, month: SupportsIndex | None = None, day: SupportsIndex | None = None
    ) -> tuple[int, int, int]: ...
    @classmethod
    def month_length(cls, year: int, month: int) -> int: ...

class HijriDate:
    year: Any
    month: Any
    day: Any
    def __init__(self, year, month, day) -> None: ...
    def weekday(self): ...

class hijri_parser(non_gregorian_parser):
    calendar_converter: type[hijri]
    default_year: int
    default_month: int
    default_day: int
    non_gregorian_date_cls: type[HijriDate]
    def handle_two_digit_year(self, year: int) -> int: ...



import ast
from typing_extensions import assert_type

# Test with source code strings
assert_type(ast.parse("x = 1"), ast.Module)
assert_type(ast.parse("x = 1", mode="exec"), ast.Module)
assert_type(ast.parse("1 + 1", mode="eval"), ast.Expression)
assert_type(ast.parse("x = 1", mode="single"), ast.Interactive)
assert_type(ast.parse("(int, str) -> None", mode="func_type"), ast.FunctionType)

# Test with mod objects - Module
mod1: ast.Module = ast.Module([], [])
assert_type(ast.parse(mod1), ast.Module)
assert_type(ast.parse(mod1, mode="exec"), ast.Module)
mod2: ast.Module = ast.Module(body=[ast.Expr(value=ast.Constant(value=42))], type_ignores=[])
assert_type(ast.parse(mod2), ast.Module)

# Test with mod objects - Expression
expr1: ast.Expression = ast.Expression(body=ast.Constant(value=42))
assert_type(ast.parse(expr1, mode="eval"), ast.Expression)

# Test with mod objects - Interactive
inter1: ast.Interactive = ast.Interactive(body=[])
assert_type(ast.parse(inter1, mode="single"), ast.Interactive)

# Test with mod objects - FunctionType
func1: ast.FunctionType = ast.FunctionType(argtypes=[], returns=ast.Constant(value=None))
assert_type(ast.parse(func1, mode="func_type"), ast.FunctionType)

# Test that any AST node can be passed and returns the same type
binop: ast.BinOp = ast.BinOp(left=ast.Constant(1), op=ast.Add(), right=ast.Constant(2))
assert_type(ast.parse(binop), ast.BinOp)

constant: ast.Constant = ast.Constant(value=42)
assert_type(ast.parse(constant), ast.Constant)

expr_stmt: ast.Expr = ast.Expr(value=ast.Constant(value=42))
assert_type(ast.parse(expr_stmt), ast.Expr)

# Test with additional parameters
assert_type(ast.parse(mod1, filename="test.py"), ast.Module)
assert_type(ast.parse(mod1, type_comments=True), ast.Module)
assert_type(ast.parse(mod1, feature_version=(3, 10)), ast.Module)
assert_type(ast.parse(binop, filename="test.py"), ast.BinOp)



class IssuerParameter:
    def __call__(self, authorization_server) -> None: ...
    def add_issuer_parameter(self, authorization_server, response) -> None: ...
    def get_issuer(self) -> str | None: ...



from grpc_channelz.v1 import _async as aio
from grpc_channelz.v1._servicer import ChannelzServicer

def add_channelz_servicer(server) -> None: ...

__all__ = ["aio", "add_channelz_servicer", "ChannelzServicer"]



from _typeshed import Incomplete

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["spectral_graph_forge"]

@_dispatchable
def spectral_graph_forge(G: Graph[_Node], alpha, transformation: str = "identity", seed=None) -> Graph[Incomplete]: ...



import abc
import asyncio
from _typeshed import Incomplete
from collections.abc import AsyncIterable, AsyncIterator, Awaitable, Callable, Generator, Iterable, Iterator, Mapping, Sequence
from concurrent import futures
from types import TracebackType
from typing import Any, Generic, NoReturn, TypeVar, overload, type_check_only
from typing_extensions import Self, TypeAlias

from grpc import (
    CallCredentials,
    ChannelConnectivity,
    ChannelCredentials,
    Compression,
    GenericRpcHandler,
    HandlerCallDetails,
    RpcError,
    RpcMethodHandler,
    ServerCredentials,
    StatusCode,
    _Options,
)

_TRequest = TypeVar("_TRequest")
_TResponse = TypeVar("_TResponse")

# Exceptions:

class BaseError(Exception): ...
class UsageError(BaseError): ...
class AbortError(BaseError): ...
class InternalError(BaseError): ...

class AioRpcError(RpcError):
    def __init__(
        self,
        code: StatusCode,
        initial_metadata: Metadata,
        trailing_metadata: Metadata,
        details: str | None = None,
        debug_error_string: str | None = None,
    ) -> None: ...
    def debug_error_string(self) -> str: ...
    def initial_metadata(self) -> Metadata: ...

# Create Client:

def insecure_channel(
    target: str,
    options: _Options | None = None,
    compression: Compression | None = None,
    interceptors: Sequence[ClientInterceptor] | None = None,
) -> Channel: ...
def secure_channel(
    target: str,
    credentials: ChannelCredentials,
    options: _Options | None = None,
    compression: Compression | None = None,
    interceptors: Sequence[ClientInterceptor] | None = None,
) -> Channel: ...

# Create Server:

def server(
    migration_thread_pool: futures.Executor | None = None,
    handlers: Sequence[GenericRpcHandler] | None = None,
    interceptors: Sequence[ServerInterceptor] | None = None,
    options: _Options | None = None,
    maximum_concurrent_rpcs: int | None = None,
    compression: Compression | None = None,
) -> Server: ...

# Channel Object:

_Serializer: TypeAlias = Callable[[_T], bytes]
_Deserializer: TypeAlias = Callable[[bytes], _T]

class Channel(abc.ABC):
    @abc.abstractmethod
    async def close(self, grace: float | None = None) -> None: ...
    @abc.abstractmethod
    def get_state(self, try_to_connect: bool = False) -> ChannelConnectivity: ...
    @abc.abstractmethod
    async def wait_for_state_change(self, last_observed_state: ChannelConnectivity) -> None: ...
    @abc.abstractmethod
    def stream_stream(
        self,
        method: str,
        request_serializer: _Serializer[_TRequest] | None = None,
        response_deserializer: _Deserializer[_TResponse] | None = None,
    ) -> StreamStreamMultiCallable[_TRequest, _TResponse]: ...
    @abc.abstractmethod
    def stream_unary(
        self,
        method: str,
        request_serializer: _Serializer[_TRequest] | None = None,
        response_deserializer: _Deserializer[_TResponse] | None = None,
    ) -> StreamUnaryMultiCallable[_TRequest, _TResponse]: ...
    @abc.abstractmethod
    def unary_stream(
        self,
        method: str,
        request_serializer: _Serializer[_TRequest] | None = None,
        response_deserializer: _Deserializer[_TResponse] | None = None,
    ) -> UnaryStreamMultiCallable[_TRequest, _TResponse]: ...
    @abc.abstractmethod
    def unary_unary(
        self,
        method: str,
        request_serializer: _Serializer[_TRequest] | None = None,
        response_deserializer: _Deserializer[_TResponse] | None = None,
    ) -> UnaryUnaryMultiCallable[_TRequest, _TResponse]: ...
    @abc.abstractmethod
    async def __aenter__(self) -> Self: ...
    @abc.abstractmethod
    async def __aexit__(
        self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None
    ) -> bool | None: ...
    @abc.abstractmethod
    async def channel_ready(self) -> None: ...

# Server Object:

class Server(metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def add_generic_rpc_handlers(self, generic_rpc_handlers: Iterable[GenericRpcHandler]) -> None: ...

    # Returns an integer port on which server will accept RPC requests.
    @abc.abstractmethod
    def add_insecure_port(self, address: str) -> int: ...

    # Returns an integer port on which server will accept RPC requests.
    @abc.abstractmethod
    def add_secure_port(self, address: str, server_credentials: ServerCredentials) -> int: ...
    @abc.abstractmethod
    async def start(self) -> None: ...

    # Grace period is in seconds.
    @abc.abstractmethod
    async def stop(self, grace: float | None) -> None: ...

    # Returns a bool indicates if the operation times out. Timeout is in seconds.
    @abc.abstractmethod
    async def wait_for_termination(self, timeout: float | None = None) -> bool: ...

# Client-Side Context:

_DoneCallbackType: TypeAlias = Callable[[Any], None]
_EOFType: TypeAlias = object

class RpcContext(metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def cancelled(self) -> bool: ...
    @abc.abstractmethod
    def done(self) -> bool: ...
    @abc.abstractmethod
    def time_remaining(self) -> float | None: ...
    @abc.abstractmethod
    def cancel(self) -> bool: ...
    @abc.abstractmethod
    def add_done_callback(self, callback: _DoneCallbackType) -> None: ...

class Call(RpcContext, metaclass=abc.ABCMeta):
    @abc.abstractmethod
    async def initial_metadata(self) -> Metadata: ...
    @abc.abstractmethod
    async def trailing_metadata(self) -> Metadata: ...
    @abc.abstractmethod
    async def code(self) -> StatusCode: ...
    @abc.abstractmethod
    async def details(self) -> str: ...
    @abc.abstractmethod
    async def wait_for_connection(self) -> None: ...

class UnaryUnaryCall(Call, Generic[_TRequest, _TResponse], metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def __await__(self) -> Generator[None, None, _TResponse]: ...

class UnaryStreamCall(Call, Generic[_TRequest, _TResponse], metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def __aiter__(self) -> AsyncIterator[_TResponse]: ...
    @abc.abstractmethod
    async def read(self) -> _EOFType | _TResponse: ...

class StreamUnaryCall(Call, Generic[_TRequest, _TResponse], metaclass=abc.ABCMeta):
    @abc.abstractmethod
    async def write(self, request: _TRequest) -> None: ...
    @abc.abstractmethod
    async def done_writing(self) -> None: ...
    @abc.abstractmethod
    def __await__(self) -> Generator[None, None, _TResponse]: ...

class StreamStreamCall(Call, Generic[_TRequest, _TResponse], metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def __aiter__(self) -> AsyncIterator[_TResponse]: ...
    @abc.abstractmethod
    async def read(self) -> _EOFType | _TResponse: ...
    @abc.abstractmethod
    async def write(self, request: _TRequest) -> None: ...
    @abc.abstractmethod
    async def done_writing(self) -> None: ...

# Service-Side Context:

@type_check_only
class _DoneCallback(Generic[_TRequest, _TResponse]):
    def __call__(self, ctx: ServicerContext[_TRequest, _TResponse]) -> None: ...

class ServicerContext(Generic[_TRequest, _TResponse], metaclass=abc.ABCMeta):
    @abc.abstractmethod
    async def abort(self, code: StatusCode, details: str = "", trailing_metadata: _MetadataType = ()) -> NoReturn: ...
    @abc.abstractmethod
    async def read(self) -> _TRequest: ...
    @abc.abstractmethod
    async def write(self, message: _TResponse) -> None: ...
    @abc.abstractmethod
    async def send_initial_metadata(self, initial_metadata: _MetadataType) -> None: ...
    def add_done_callback(self, callback: _DoneCallback[_TRequest, _TResponse]) -> None: ...
    @abc.abstractmethod
    def set_trailing_metadata(self, trailing_metadata: _MetadataType) -> None: ...
    @abc.abstractmethod
    def invocation_metadata(self) -> Metadata | None: ...
    @abc.abstractmethod
    def set_code(self, code: StatusCode) -> None: ...
    @abc.abstractmethod
    def set_details(self, details: str) -> None: ...
    @abc.abstractmethod
    def set_compression(self, compression: Compression) -> None: ...
    @abc.abstractmethod
    def disable_next_message_compression(self) -> None: ...
    @abc.abstractmethod
    def peer(self) -> str: ...
    @abc.abstractmethod
    def peer_identities(self) -> Iterable[bytes] | None: ...
    @abc.abstractmethod
    def peer_identity_key(self) -> str | None: ...
    @abc.abstractmethod
    def auth_context(self) -> Mapping[str, Iterable[bytes]]: ...
    def time_remaining(self) -> float: ...
    def trailing_metadata(self) -> Metadata: ...
    def code(self) -> StatusCode: ...
    def details(self) -> str: ...
    def cancelled(self) -> bool: ...
    def done(self) -> bool: ...

# Client-Side Interceptor:

class ClientCallDetails(abc.ABC):
    def __new__(
        _cls,
        method: str,
        timeout: float | None,
        metadata: Metadata | None,
        credentials: CallCredentials | None,
        wait_for_ready: bool | None,
    ) -> Self: ...

    method: str
    timeout: float | None
    metadata: Metadata | None
    credentials: CallCredentials | None

    # "This is an EXPERIMENTAL argument. An optional flag t enable wait for ready mechanism."
    wait_for_ready: bool | None

    # As at 1.53.0, this is not supported in aio:
    # compression: Compression | None

@type_check_only
class _InterceptedCall(Generic[_TRequest, _TResponse]):
    def __init__(self, interceptors_task: asyncio.Task[Any]) -> None: ...
    def __del__(self) -> None: ...
    def cancel(self) -> bool: ...
    def cancelled(self) -> bool: ...
    def done(self) -> bool: ...
    def add_done_callback(self, callback: _DoneCallback[_TRequest, _TResponse]) -> None: ...
    def time_remaining(self) -> float | None: ...
    async def initial_metadata(self) -> Metadata | None: ...
    async def trailing_metadata(self) -> Metadata | None: ...
    async def code(self) -> StatusCode: ...
    async def details(self) -> str: ...
    async def debug_error_string(self) -> str | None: ...
    async def wait_for_connection(self) -> None: ...

class InterceptedUnaryUnaryCall(_InterceptedCall[_TRequest, _TResponse], metaclass=abc.ABCMeta):
    def __await__(self) -> Generator[Incomplete, None, _TResponse]: ...
    def __init__(
        self,
        interceptors: Sequence[UnaryUnaryClientInterceptor],
        request: _TRequest,
        timeout: float | None,
        metadata: Metadata,
        credentials: CallCredentials | None,
        wait_for_ready: bool | None,
        channel: Channel,
        method: bytes,
        request_serializer: _Serializer[_TRequest],
        response_deserializer: _Deserializer[_TResponse],
        loop: asyncio.AbstractEventLoop,
    ) -> None: ...

    # pylint: disable=too-many-arguments
    async def _invoke(
        self,
        interceptors: Sequence[UnaryUnaryClientInterceptor],
        method: bytes,
        timeout: float | None,
        metadata: Metadata | None,
        credentials: CallCredentials | None,
        wait_for_ready: bool | None,
        request: _TRequest,
        request_serializer: _Serializer[_TRequest],
        response_deserializer: _Deserializer[_TResponse],
    ) -> UnaryUnaryCall[_TRequest, _TResponse]: ...
    def time_remaining(self) -> float | None: ...

class ClientInterceptor(metaclass=abc.ABCMeta): ...

class UnaryUnaryClientInterceptor(ClientInterceptor, metaclass=abc.ABCMeta):
    # This method (not the class) is generic over _TRequest and _TResponse
    # and the types must satisfy the no-op implementation of
    # `return await continuation(client_call_details, request)`.
    @abc.abstractmethod
    async def intercept_unary_unary(
        self,
        continuation: Callable[[ClientCallDetails, _TRequest], Awaitable[UnaryUnaryCall[_TRequest, _TResponse]]],
        client_call_details: ClientCallDetails,
        request: _TRequest,
    ) -> _TResponse | UnaryUnaryCall[_TRequest, _TResponse]: ...

class UnaryStreamClientInterceptor(ClientInterceptor, metaclass=abc.ABCMeta):
    # This method (not the class) is generic over _TRequest and _TResponse
    # and the types must satisfy the no-op implementation of
    # `return await continuation(client_call_details, request)`.
    @abc.abstractmethod
    async def intercept_unary_stream(
        self,
        continuation: Callable[[ClientCallDetails, _TRequest], Awaitable[UnaryStreamCall[_TRequest, _TResponse]]],
        client_call_details: ClientCallDetails,
        request: _TRequest,
    ) -> AsyncIterator[_TResponse] | UnaryStreamCall[_TRequest, _TResponse]: ...

class StreamUnaryClientInterceptor(ClientInterceptor, metaclass=abc.ABCMeta):
    # This method (not the class) is generic over _TRequest and _TResponse
    # and the types must satisfy the no-op implementation of
    # `return await continuation(client_call_details, request_iterator)`.
    @abc.abstractmethod
    async def intercept_stream_unary(
        self,
        continuation: Callable[
            [ClientCallDetails, AsyncIterable[_TRequest] | Iterable[_TRequest]], Awaitable[StreamUnaryCall[_TRequest, _TResponse]]
        ],
        client_call_details: ClientCallDetails,
        request_iterator: AsyncIterable[_TRequest] | Iterable[_TRequest],
    ) -> _TResponse | StreamUnaryCall[_TRequest, _TResponse]: ...

class StreamStreamClientInterceptor(ClientInterceptor, metaclass=abc.ABCMeta):
    # This method (not the class) is generic over _TRequest and _TResponse
    # and the types must satisfy the no-op implementation of
    # `return await continuation(client_call_details, request_iterator)`.
    @abc.abstractmethod
    async def intercept_stream_stream(
        self,
        continuation: Callable[
            [ClientCallDetails, AsyncIterable[_TRequest] | Iterable[_TRequest]],
            Awaitable[StreamStreamCall[_TRequest, _TResponse]],
        ],
        client_call_details: ClientCallDetails,
        request_iterator: AsyncIterable[_TRequest] | Iterable[_TRequest],
    ) -> AsyncIterator[_TResponse] | StreamStreamCall[_TRequest, _TResponse]: ...

# Server-Side Interceptor:

class ServerInterceptor(metaclass=abc.ABCMeta):
    # This method (not the class) is generic over _TRequest and _TResponse
    # and the types must satisfy the no-op implementation of
    # `return await continuation(handler_call_details)`.
    @abc.abstractmethod
    async def intercept_service(
        self,
        continuation: Callable[[HandlerCallDetails], Awaitable[RpcMethodHandler[_TRequest, _TResponse]]],
        handler_call_details: HandlerCallDetails,
    ) -> RpcMethodHandler[_TRequest, _TResponse]: ...

# Multi-Callable Interfaces:

class UnaryUnaryMultiCallable(Generic[_TRequest, _TResponse], metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def __call__(
        self,
        request: _TRequest,
        *,
        timeout: float | None = None,
        metadata: _MetadataType | None = None,
        credentials: CallCredentials | None = None,
        wait_for_ready: bool | None = None,
        compression: Compression | None = None,
    ) -> UnaryUnaryCall[_TRequest, _TResponse]: ...

class UnaryStreamMultiCallable(Generic[_TRequest, _TResponse], metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def __call__(
        self,
        request: _TRequest,
        *,
        timeout: float | None = None,
        metadata: _MetadataType | None = None,
        credentials: CallCredentials | None = None,
        wait_for_ready: bool | None = None,
        compression: Compression | None = None,
    ) -> UnaryStreamCall[_TRequest, _TResponse]: ...

class StreamUnaryMultiCallable(Generic[_TRequest, _TResponse], metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def __call__(
        self,
        request_iterator: AsyncIterator[_TRequest] | Iterator[_TRequest] | None = None,
        timeout: float | None = None,
        metadata: _MetadataType | None = None,
        credentials: CallCredentials | None = None,
        wait_for_ready: bool | None = None,
        compression: Compression | None = None,
    ) -> StreamUnaryCall[_TRequest, _TResponse]: ...

class StreamStreamMultiCallable(Generic[_TRequest, _TResponse], metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def __call__(
        self,
        request_iterator: AsyncIterator[_TRequest] | Iterator[_TRequest] | None = None,
        timeout: float | None = None,
        metadata: _MetadataType | None = None,
        credentials: CallCredentials | None = None,
        wait_for_ready: bool | None = None,
        compression: Compression | None = None,
    ) -> StreamStreamCall[_TRequest, _TResponse]: ...

# Metadata:

_MetadataKey: TypeAlias = str
_MetadataValue: TypeAlias = str | bytes
_MetadatumType: TypeAlias = tuple[_MetadataKey, _MetadataValue]
_MetadataType: TypeAlias = Metadata | Sequence[_MetadatumType]
_T = TypeVar("_T")

class Metadata(Mapping[_MetadataKey, _MetadataValue]):
    def __init__(self, *args: tuple[_MetadataKey, _MetadataValue]) -> None: ...
    @classmethod
    def from_tuple(cls, raw_metadata: tuple[_MetadataKey, _MetadataValue]) -> Metadata: ...
    def add(self, key: _MetadataKey, value: _MetadataValue) -> None: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: _MetadataKey) -> _MetadataValue: ...
    def __setitem__(self, key: _MetadataKey, value: _MetadataValue) -> None: ...
    def __delitem__(self, key: _MetadataKey) -> None: ...
    def delete_all(self, key: _MetadataKey) -> None: ...
    def __iter__(self) -> Iterator[_MetadataKey]: ...
    @overload
    def get(self, key: _MetadataKey, default: None = None) -> _MetadataValue | None: ...
    @overload
    def get(self, key: _MetadataKey, default: _MetadataValue) -> _MetadataValue: ...
    @overload
    def get(self, key: _MetadataKey, default: _T) -> _MetadataValue | _T: ...
    def get_all(self, key: _MetadataKey) -> list[_MetadataValue]: ...
    def set_all(self, key: _MetadataKey, values: list[_MetadataValue]) -> None: ...
    def __contains__(self, key: object) -> bool: ...
    def __eq__(self, other: object) -> bool: ...
    def __add__(self, other: Any) -> Metadata: ...



from _typeshed import FileDescriptorOrPath, StrPath
from collections.abc import Iterator
from typing import Any

from pygments.lexer import Lexer, LexerMeta

def get_all_lexers(plugins: bool = True) -> Iterator[tuple[str, tuple[str, ...], tuple[str, ...], tuple[str, ...]]]: ...
def find_lexer_class(name: str) -> LexerMeta | None: ...
def find_lexer_class_by_name(_alias: str) -> LexerMeta: ...
def get_lexer_by_name(_alias: str, **options: Any) -> Lexer: ...
def load_lexer_from_file(filename: FileDescriptorOrPath, lexername: str = "CustomLexer", **options: Any) -> Lexer: ...
def find_lexer_class_for_filename(_fn: StrPath, code: str | bytes | None = None) -> LexerMeta | None: ...
def get_lexer_for_filename(_fn: StrPath, code: str | bytes | None = None, **options: Any) -> Lexer: ...
def get_lexer_for_mimetype(_mime: str, **options: Any) -> Lexer: ...
def guess_lexer_for_filename(_fn: StrPath, _text: str, **options: Any) -> Lexer: ...
def guess_lexer(_text: str | bytes, **options: Any) -> Lexer: ...

# Having every lexer class here doesn't seem to be worth it
def __getattr__(name: str): ...  # incomplete module



from _typeshed import Incomplete
from collections.abc import Iterable

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["node_redundancy"]

@_dispatchable
def node_redundancy(G: Graph[_Node], nodes: Iterable[Incomplete] | None = None) -> dict[Incomplete, float]: ...



from _typeshed import Incomplete
from collections.abc import Mapping
from typing import Final

SQS_XRAY_HEADER: Final = "AWSTraceHeader"

class SqsMessageHelper:
    @staticmethod
    def isSampled(sqs_message: Mapping[str, Incomplete]) -> bool: ...



from typing import ClassVar

from .core import UnitedStates

class California(UnitedStates):
    shift_exceptions: ClassVar[list[tuple[int, int]]]

class CaliforniaEducation(California): ...
class CaliforniaBerkeley(California): ...
class CaliforniaSanFrancisco(California): ...
class CaliforniaWestHollywood(California): ...



from _typeshed import Incomplete
from collections.abc import Mapping, MutableMapping, Sequence
from re import Pattern
from typing import Any, Final, Literal, TypedDict, overload, type_check_only
from typing_extensions import Required, TypeAlias, deprecated

import requests
from requests.models import Request, Response
from requests.sessions import _Auth

LAUNCHER_SSH: Final[str]
LAUNCHER_COMMAND: Final[str]
LAUNCHER_JNLP: Final[str]
LAUNCHER_WINDOWS_SERVICE: Final[str]
DEFAULT_HEADERS: Final[dict[str, str]]
DEFAULT_TIMEOUT: Final[float]
DEFAULT_RETRIES: Final = 0
INFO: Final[str]
PLUGIN_INFO: Final[str]
CRUMB_URL: Final[str]
WHOAMI_URL: Final[str]
JOBS_QUERY: Final[str]
JOBS_QUERY_TREE: Final[str]
JOB_INFO: Final[str]
JOB_NAME: Final[str]
ALL_BUILDS: Final[str]
Q_INFO: Final[str]
Q_ITEM: Final[str]
CANCEL_QUEUE: Final[str]
CREATE_JOB: Final[str]
CONFIG_JOB: Final[str]
DELETE_JOB: Final[str]
ENABLE_JOB: Final[str]
DISABLE_JOB: Final[str]
CHECK_JENKINSFILE_SYNTAX: Final[str]
SET_JOB_BUILD_NUMBER: Final[str]
COPY_JOB: Final[str]
RENAME_JOB: Final[str]
BUILD_JOB: Final[str]
STOP_BUILD: Final[str]
BUILD_WITH_PARAMS_JOB: Final[str]
BUILD_INFO: Final[str]
BUILD_CONSOLE_OUTPUT: Final[str]
BUILD_ENV_VARS: Final[str]
BUILD_TEST_REPORT: Final[str]
BUILD_ARTIFACT: Final[str]
BUILD_STAGES: Final[str]
DELETE_BUILD: Final[str]
WIPEOUT_JOB_WORKSPACE: Final[str]
NODE_LIST: Final[str]
CREATE_NODE: Final[str]
DELETE_NODE: Final[str]
NODE_INFO: Final[str]
NODE_TYPE: Final[str]
TOGGLE_OFFLINE: Final[str]
CONFIG_NODE: Final[str]
VIEW_NAME: Final[str]
VIEW_JOBS: Final[str]
CREATE_VIEW: Final[str]
CONFIG_VIEW: Final[str]
DELETE_VIEW: Final[str]
SCRIPT_TEXT: Final[str]
NODE_SCRIPT_TEXT: Final[str]
PROMOTION_NAME: Final[str]
PROMOTION_INFO: Final[str]
DELETE_PROMOTION: Final[str]
CREATE_PROMOTION: Final[str]
CONFIG_PROMOTION: Final[str]
LIST_CREDENTIALS: Final[str]
CREATE_CREDENTIAL: Final[str]
CONFIG_CREDENTIAL: Final[str]
CREDENTIAL_INFO: Final[str]
QUIET_DOWN: Final[str]
EMPTY_CONFIG_XML: Final[str]
EMPTY_FOLDER_XML: Final[str]
RECONFIG_XML: Final[str]
EMPTY_VIEW_CONFIG_XML: Final[str]
EMPTY_PROMO_CONFIG_XML: Final[str]
PROMO_RECONFIG_XML: Final[str]

class JenkinsException(Exception): ...
class NotFoundException(JenkinsException): ...
class EmptyResponseException(JenkinsException): ...
class BadHTTPException(JenkinsException): ...
class TimeoutException(JenkinsException): ...

class WrappedSession(requests.Session):
    # merge_environment_settings wraps requests.Session.merge_environment_settings
    # w/o changing the type signature
    ...

_JSONValue: TypeAlias = Any  # too many possibilities to express
_JSON: TypeAlias = dict[str, _JSONValue]

@type_check_only
class _Job(TypedDict, total=False):
    _class: Required[str]
    url: Required[str]
    color: str
    name: Required[str]
    fullname: Required[str]
    jobs: list[_Job]

class Jenkins:
    server: str
    auth: _Auth | None
    crumb: Mapping[str, Incomplete] | bool | Incomplete
    timeout: int
    def __init__(
        self, url: str, username: str | None = None, password: str | None = None, timeout: int = ..., retries: int = 0
    ) -> None: ...
    def maybe_add_crumb(self, req: Request) -> None: ...
    def get_job_info(self, name: str, depth: int = 0, fetch_all_builds: bool = False) -> _JSON: ...
    def get_job_info_regex(
        self, pattern: str | Pattern[str], depth: int = 0, folder_depth: int = 0, folder_depth_per_request: int = 10
    ) -> list[_JSON]: ...
    def get_job_name(self, name: str) -> str | None: ...
    def debug_job_info(self, job_name: str) -> None: ...
    def jenkins_open(self, req: Request, add_crumb: bool = True, resolve_auth: bool = True) -> str: ...
    def jenkins_open_stream(self, req: Request, add_crumb: bool = True, resolve_auth: bool = True) -> Response: ...
    def jenkins_request(
        self, req: Request, add_crumb: bool = True, resolve_auth: bool = True, stream: bool | None = None
    ) -> Response: ...
    def get_queue_item(self, number: int, depth: int = 0) -> _JSON: ...
    def get_build_info(self, name: str, number: int, depth: int = 0) -> _JSON: ...
    def get_build_env_vars(self, name: str, number: int, depth: int = 0) -> _JSON | None: ...
    def get_build_test_report(self, name: str, number: int, depth: int = 0, tree: str | None = None) -> _JSON | None: ...
    def get_build_artifact(self, name: str, number: int, artifact: str) -> _JSON: ...
    def get_build_artifact_as_bytes(self, name: str, number: int, artifact: str) -> bytes: ...
    def get_build_stages(self, name: str, number: int) -> _JSON: ...
    def get_queue_info(self) -> _JSON: ...
    def cancel_queue(self, id: int) -> None: ...
    def get_info(self, item: str = "", query: str | None = None) -> _JSON: ...
    def get_whoami(self, depth: int = 0) -> _JSON: ...
    def get_version(self) -> str: ...
    @deprecated("Deprecated since 0.4.9. Use `get_plugins` instead.")
    def get_plugins_info(self, depth: int = 2) -> _JSON: ...
    def get_plugin_info(self, name: str, depth: int = 2) -> _JSON: ...
    def get_plugins(self, depth: int = 2) -> _JSON: ...
    def get_jobs(self, folder_depth: int = 0, folder_depth_per_request: int = 10, view_name: str | None = None) -> list[_Job]: ...
    def get_all_jobs(self, folder_depth: int | None = None, folder_depth_per_request: int = 10) -> list[_Job]: ...
    def copy_job(self, from_name: str, to_name: str) -> None: ...
    def rename_job(self, from_name: str, to_name: str) -> None: ...
    def delete_job(self, name: str) -> None: ...
    def enable_job(self, name: str) -> None: ...
    def disable_job(self, name: str) -> None: ...
    def set_next_build_number(self, name: str, number: int) -> None: ...
    def job_exists(self, name: str) -> bool: ...
    def jobs_count(self) -> int: ...
    def assert_job_exists(self, name: str, exception_message: str = "job[%s] does not exist") -> None: ...
    def create_folder(self, folder_name: str, ignore_failures: bool = False) -> None: ...
    def upsert_job(self, name: str, config_xml: str) -> None: ...
    def check_jenkinsfile_syntax(self, jenkinsfile: str) -> list[str]: ...
    def create_job(self, name: str, config_xml: str) -> None: ...
    def get_job_config(self, name: str) -> str: ...
    def reconfig_job(self, name: str, config_xml: str) -> None: ...
    @overload
    def build_job_url(
        self,
        name: str,
        parameters: Mapping[str, Incomplete] | Sequence[tuple[str, Incomplete]] | None = None,
        token: Literal[""] | None = None,
    ) -> str: ...
    @overload
    def build_job_url(
        self, name: str, parameters: dict[str, Incomplete] | list[tuple[str, Incomplete]] | None, token: str
    ) -> str: ...
    @overload
    def build_job_url(
        self, name: str, parameters: dict[str, Incomplete] | list[tuple[str, Incomplete]] | None = None, *, token: str
    ) -> str: ...
    @overload
    def build_job(
        self,
        name: str,
        parameters: Mapping[str, Incomplete] | Sequence[tuple[str, Incomplete]] | None = None,
        token: Literal[""] | None = None,
    ) -> int: ...
    @overload
    def build_job(
        self, name: str, parameters: dict[str, Incomplete] | list[tuple[str, Incomplete]] | None, token: str
    ) -> int: ...
    @overload
    def build_job(
        self, name: str, parameters: dict[str, Incomplete] | list[tuple[str, Incomplete]] | None = None, *, token: str
    ) -> int: ...
    def run_script(self, script: str, node: str | None = None) -> str: ...
    def install_plugin(self, name: str, include_dependencies: bool = True) -> bool: ...
    def stop_build(self, name: str, number: int) -> None: ...
    def delete_build(self, name: str, number: int) -> None: ...
    def wipeout_job_workspace(self, name: str) -> None: ...
    def get_running_builds(self) -> list[_JSON]: ...
    def get_nodes_with_info(self, depth: int = 0) -> list[_JSON]: ...
    def get_nodes(self, depth: int = 0) -> list[_JSON]: ...
    def get_node_info(self, name: str, depth: int = 0) -> _JSON: ...
    def node_exists(self, name: str) -> bool: ...
    def assert_node_exists(self, name: str, exception_message: str = "node[%s] does not exist") -> None: ...
    def delete_node(self, name: str) -> None: ...
    def disable_node(self, name: str, msg: str = "") -> None: ...
    def enable_node(self, name: str) -> None: ...
    def create_node(
        self,
        name: str,
        numExecutors: int = 2,
        nodeDescription: str | None = None,
        remoteFS: str = "/var/lib/jenkins",
        labels: str | None = None,
        exclusive: bool = False,
        launcher: str = "hudson.slaves.CommandLauncher",
        launcher_params: MutableMapping[str, Incomplete] = {},
    ) -> None: ...
    def get_node_config(self, name: str) -> str: ...
    def reconfig_node(self, name: str, config_xml: str) -> None: ...
    def get_build_console_output(self, name: str, number: int) -> str: ...
    def get_view_name(self, name: str) -> str | None: ...
    def assert_view_exists(self, name: str, exception_message: str = "view[%s] does not exist") -> None: ...
    def view_exists(self, name: str) -> bool: ...
    def get_views(self) -> list[_JSON]: ...
    def delete_view(self, name: str) -> None: ...
    def create_view(self, name: str, config_xml: str) -> None: ...
    def reconfig_view(self, name: str, config_xml: str) -> None: ...
    def get_view_config(self, name: str) -> str: ...
    def get_promotion_name(self, name: str, job_name: str) -> str | None: ...
    def assert_promotion_exists(
        self, name: str, job_name: str, exception_message: str = "promotion[%s] does not exist for job[%s]"
    ) -> None: ...
    def promotion_exists(self, name: str, job_name: str) -> bool: ...
    def get_promotions_info(self, job_name: str, depth: int = 0) -> _JSON: ...
    def get_promotions(self, job_name: str) -> list[_JSON]: ...
    def delete_promotion(self, name: str, job_name: str) -> None: ...
    def create_promotion(self, name: str, job_name: str, config_xml: str) -> None: ...
    def reconfig_promotion(self, name: str, job_name: str, config_xml: str) -> None: ...
    def get_promotion_config(self, name: str, job_name: str) -> str: ...
    def assert_folder(self, name: str, exception_message: str = "job[%s] is not a folder") -> None: ...
    def is_folder(self, name: str) -> bool: ...
    def assert_credential_exists(
        self,
        name: str,
        folder_name: str,
        domain_name: str = "_",
        exception_message: str = "credential[%s] does not exist in the domain[%s] of [%s]",  # noqa: Y053
    ) -> None: ...
    def credential_exists(self, name: str, folder_name: str, domain_name: str = "_") -> bool: ...
    def get_credential_info(self, name: str, folder_name: str, domain_name: str = "_") -> _JSON: ...
    def get_credential_config(self, name: str, folder_name: str, domain_name: str = "_") -> str: ...
    def create_credential(self, folder_name: str, config_xml: str, domain_name: str = "_") -> None: ...
    def delete_credential(self, name: str, folder_name: str, domain_name: str = "_") -> None: ...
    def reconfig_credential(self, folder_name: str, config_xml: str, domain_name: str = "_") -> None: ...
    def list_credentials(self, folder_name: str, domain_name: str = "_") -> list[Incomplete]: ...
    def quiet_down(self) -> None: ...
    def wait_for_normal_op(self, timeout: int) -> bool: ...



import re
from xml.etree.ElementTree import Element

__all__ = ["to_html_string", "to_xhtml_string"]

RE_AMP: re.Pattern[str]

def to_html_string(element: Element) -> str: ...
def to_xhtml_string(element: Element) -> str: ...



from _typeshed import Incomplete

from pony.orm.core import Database, Entity

db: Database

class Customer(Entity):
    __slots__ = ()
    id: Incomplete
    name: Incomplete
    email: Incomplete
    orders: Incomplete

class Order(Entity):
    __slots__ = ()
    id: Incomplete
    total_price: Incomplete
    customer: Incomplete
    items: Incomplete

class Product(Entity):
    __slots__ = ()
    id: Incomplete
    name: Incomplete
    price: Incomplete
    items: Incomplete

class OrderItem(Entity):
    __slots__ = ()
    quantity: Incomplete
    order: Incomplete
    product: Incomplete

def populate_database() -> None: ...



import datetime
from typing import ClassVar

from ..core import ChineseNewYearCalendar, IslamicMixin, WesternMixin

class Singapore(WesternMixin, IslamicMixin, ChineseNewYearCalendar):
    DEEPAVALI: ClassVar[dict[int, datetime.date]]



from _typeshed import Incomplete

from reportlab.graphics.barcode.common import Barcode

class ECC200DataMatrix(Barcode):
    barWidth: int
    row_modules: int
    col_modules: int
    row_regions: int
    col_regions: int
    cw_data: int
    cw_ecc: int
    row_usable_modules: Incomplete
    col_usable_modules: Incomplete
    def __init__(self, *args, **kwargs) -> None: ...
    valid: int
    validated: Incomplete
    def validate(self) -> None: ...
    encoded: Incomplete
    def encode(self): ...
    def computeSize(self, *args) -> None: ...
    def draw(self) -> None: ...

__all__ = ("ECC200DataMatrix",)



from _typeshed import Incomplete
from collections.abc import Iterable

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = [
    "group_betweenness_centrality",
    "group_closeness_centrality",
    "group_degree_centrality",
    "group_in_degree_centrality",
    "group_out_degree_centrality",
    "prominent_group",
]

@_dispatchable
def group_betweenness_centrality(
    G: Graph[_Node], C, normalized: bool | None = True, weight: str | None = None, endpoints: bool | None = False
): ...
@_dispatchable
def prominent_group(
    G: Graph[_Node],
    k: int,
    weight: str | None = None,
    C: Iterable[Incomplete] | None = None,
    endpoints: bool | None = False,
    normalized: bool | None = True,
    greedy: bool | None = False,
) -> tuple[float, list[Incomplete]]: ...
@_dispatchable
def group_closeness_centrality(G: Graph[_Node], S: Iterable[Incomplete], weight: str | None = None) -> float: ...
@_dispatchable
def group_degree_centrality(G: Graph[_Node], S: Iterable[Incomplete]) -> float: ...
@_dispatchable
def group_in_degree_centrality(G: Graph[_Node], S: Iterable[Incomplete]) -> float: ...
@_dispatchable
def group_out_degree_centrality(G: Graph[_Node], S: Iterable[Incomplete]) -> float: ...



from _typeshed import Incomplete

from auth0.rest import RestClient
from auth0.types import TimeoutType

class Users:
    domain: str
    protocol: str
    client: RestClient
    def __init__(self, domain: str, telemetry: bool = True, timeout: TimeoutType = 5.0, protocol: str = "https") -> None: ...
    def userinfo(self, access_token: str) -> dict[str, Incomplete]: ...



from antlr4.InputStream import InputStream
from antlr4.Lexer import TokenSource
from antlr4.Token import CommonToken as CommonToken

class TokenFactory: ...

class CommonTokenFactory(TokenFactory):
    __slots__ = "copyText"
    DEFAULT: CommonTokenFactory | None
    copyText: bool
    def __init__(self, copyText: bool = False) -> None: ...
    def create(
        self,
        source: tuple[TokenSource, InputStream],
        type: int,
        text: str,
        channel: int,
        start: int,
        stop: int,
        line: int,
        column: int,
    ) -> CommonToken: ...
    def createThin(self, type: int, text: str) -> CommonToken: ...



from .ec_key import ECKey as ECKey
from .jwe_algs import AESAlgorithm as AESAlgorithm, ECDHESAlgorithm as ECDHESAlgorithm, u32be_len_input as u32be_len_input
from .jwe_encs import CBCHS2EncAlgorithm as CBCHS2EncAlgorithm
from .oct_key import OctKey as OctKey
from .rsa_key import RSAKey as RSAKey

__all__ = [
    "register_jws_rfc7518",
    "register_jwe_rfc7518",
    "OctKey",
    "RSAKey",
    "ECKey",
    "u32be_len_input",
    "AESAlgorithm",
    "ECDHESAlgorithm",
    "CBCHS2EncAlgorithm",
]

def register_jws_rfc7518(cls) -> None: ...
def register_jwe_rfc7518(cls) -> None: ...



SECRET_KEY = "1"

INSTALLED_APPS = (
    "django.contrib.contenttypes",
    "django.contrib.sites",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.admin.apps.SimpleAdminConfig",
    "django.contrib.staticfiles",
    "django.contrib.auth",
    "channels",
)



from __future__ import annotations

from collections.abc import Callable
from concurrent.futures.thread import ThreadPoolExecutor
from typing import Awaitable, TypeVar

import grpc
import grpc.aio

RequestT = TypeVar("RequestT")
ResponseT = TypeVar("ResponseT")


class NoopInterceptor(grpc.ServerInterceptor):
    def intercept_service(
        self,
        continuation: Callable[[grpc.HandlerCallDetails], grpc.RpcMethodHandler[RequestT, ResponseT] | None],
        handler_call_details: grpc.HandlerCallDetails,
    ) -> grpc.RpcMethodHandler[RequestT, ResponseT] | None:
        return continuation(handler_call_details)


grpc.server(interceptors=[NoopInterceptor()], thread_pool=ThreadPoolExecutor())


class NoopAioInterceptor(grpc.aio.ServerInterceptor):
    async def intercept_service(
        self,
        continuation: Callable[[grpc.HandlerCallDetails], Awaitable[grpc.RpcMethodHandler[RequestT, ResponseT]]],
        handler_call_details: grpc.HandlerCallDetails,
    ) -> grpc.RpcMethodHandler[RequestT, ResponseT]:
        return await continuation(handler_call_details)


grpc.aio.server(interceptors=[NoopAioInterceptor()])



import urllib3
import urllib3.connection
from docker.transport.basehttpadapter import BaseHTTPAdapter
from docker.transport.npipesocket import NpipeSocket
from urllib3._collections import RecentlyUsedContainer as urllib3_RecentlyUsedContainer

RecentlyUsedContainer = urllib3_RecentlyUsedContainer

class NpipeHTTPConnection(urllib3.connection.HTTPConnection):
    npipe_path: str
    timeout: int
    def __init__(self, npipe_path: str, timeout: int = 60) -> None: ...
    sock: NpipeSocket | None
    def connect(self) -> None: ...

class NpipeHTTPConnectionPool(urllib3.connectionpool.HTTPConnectionPool):
    npipe_path: str
    timeout: urllib3.Timeout
    def __init__(self, npipe_path: str, timeout: int = 60, maxsize: int = 10) -> None: ...

class NpipeHTTPAdapter(BaseHTTPAdapter):
    __attrs__: list[str]
    npipe_path: str
    timeout: int
    max_pool_size: int
    pools: RecentlyUsedContainer
    def __init__(self, base_url: str, timeout: int = 60, pool_connections: int = 25, max_pool_size: int = 10) -> None: ...
    def get_connection(self, url, proxies=None): ...
    def request_url(self, request, proxies): ...



from types import FrameType
from typing import Any
from typing_extensions import TypeAlias

from gunicorn.workers.base import Worker

IOLoop: TypeAlias = Any  # tornado IOLoop class
PeriodicCallback: TypeAlias = Any  # tornado PeriodicCallback class

TORNADO5: bool

class TornadoWorker(Worker):
    alive: bool
    server_alive: bool
    ioloop: IOLoop
    callbacks: list[PeriodicCallback]

    @classmethod
    def setup(cls) -> None: ...
    def handle_exit(self, sig: int, frame: FrameType | None) -> None: ...
    def handle_request(self) -> None: ...
    def watchdog(self) -> None: ...
    def heartbeat(self) -> None: ...
    def init_process(self) -> None: ...
    def run(self) -> None: ...



from typing import Union
from typing_extensions import assert_type

from dateutil.rrule import rrule, rruleset, rrulestr

rs1 = rrulestr("", forceset=True)
assert_type(rs1, rruleset)

rs2 = rrulestr("", compatible=True)
assert_type(rs2, rruleset)

rs3 = rrulestr("")
assert_type(rs3, Union[rrule, rruleset])



import enum
import io
import threading
from _typeshed import ConvertibleToFloat, FileDescriptorOrPath, Incomplete, StrOrBytesPath, SupportsWrite
from collections.abc import Callable
from socket import AF_INET6 as AF_INET6, AddressFamily, SocketKind
from typing import BinaryIO, Final, NamedTuple, SupportsIndex, TypeVar, overload
from typing_extensions import ParamSpec

POSIX: Final[bool]
WINDOWS: Final[bool]
LINUX: Final[bool]
MACOS: Final[bool]
OSX: Final[bool]
FREEBSD: Final[bool]
OPENBSD: Final[bool]
NETBSD: Final[bool]
BSD: Final[bool]
SUNOS: Final[bool]
AIX: Final[bool]

STATUS_RUNNING: Final = "running"
STATUS_SLEEPING: Final = "sleeping"
STATUS_DISK_SLEEP: Final = "disk-sleep"
STATUS_STOPPED: Final = "stopped"
STATUS_TRACING_STOP: Final = "tracing-stop"
STATUS_ZOMBIE: Final = "zombie"
STATUS_DEAD: Final = "dead"
STATUS_WAKE_KILL: Final = "wake-kill"
STATUS_WAKING: Final = "waking"
STATUS_IDLE: Final = "idle"
STATUS_LOCKED: Final = "locked"
STATUS_WAITING: Final = "waiting"
STATUS_SUSPENDED: Final = "suspended"
STATUS_PARKED: Final = "parked"

CONN_ESTABLISHED: Final = "ESTABLISHED"
CONN_SYN_SENT: Final = "SYN_SENT"
CONN_SYN_RECV: Final = "SYN_RECV"
CONN_FIN_WAIT1: Final = "FIN_WAIT1"
CONN_FIN_WAIT2: Final = "FIN_WAIT2"
CONN_TIME_WAIT: Final = "TIME_WAIT"
CONN_CLOSE: Final = "CLOSE"
CONN_CLOSE_WAIT: Final = "CLOSE_WAIT"
CONN_LAST_ACK: Final = "LAST_ACK"
CONN_LISTEN: Final = "LISTEN"
CONN_CLOSING: Final = "CLOSING"
CONN_NONE: Final = "NONE"

class NicDuplex(enum.IntEnum):
    NIC_DUPLEX_FULL = 2
    NIC_DUPLEX_HALF = 1
    NIC_DUPLEX_UNKNOWN = 0

NIC_DUPLEX_FULL: Final = NicDuplex.NIC_DUPLEX_FULL
NIC_DUPLEX_HALF: Final = NicDuplex.NIC_DUPLEX_HALF
NIC_DUPLEX_UNKNOWN: Final = NicDuplex.NIC_DUPLEX_UNKNOWN

class BatteryTime(enum.IntEnum):
    POWER_TIME_UNKNOWN = -1
    POWER_TIME_UNLIMITED = -2

POWER_TIME_UNKNOWN: Final = BatteryTime.POWER_TIME_UNKNOWN
POWER_TIME_UNLIMITED: Final = BatteryTime.POWER_TIME_UNLIMITED

ENCODING: Final[str]
ENCODING_ERRS: Final[str]

class sswap(NamedTuple):
    total: int
    used: int
    free: int
    percent: float
    sin: int
    sout: int

class sdiskusage(NamedTuple):
    total: int
    used: int
    free: int
    percent: float

class sdiskio(NamedTuple):
    read_count: int
    write_count: int
    read_bytes: int
    write_bytes: int
    read_time: int
    write_time: int

class sdiskpart(NamedTuple):
    device: str
    mountpoint: str
    fstype: str
    opts: str

class snetio(NamedTuple):
    bytes_sent: int
    bytes_recv: int
    packets_sent: int
    packets_recv: int
    errin: int
    errout: int
    dropin: int
    dropout: int

class suser(NamedTuple):
    name: str
    terminal: str | None
    host: str | None
    started: float
    pid: str

class sconn(NamedTuple):
    fd: int
    family: AddressFamily
    type: SocketKind
    laddr: addr | tuple[()]
    raddr: addr | tuple[()]
    status: str
    pid: int | None

class snicaddr(NamedTuple):
    family: AddressFamily
    address: str
    netmask: str | None
    broadcast: str | None
    ptp: str | None

class snicstats(NamedTuple):
    isup: bool
    duplex: int
    speed: int
    mtu: int
    flags: str

class scpustats(NamedTuple):
    ctx_switches: int
    interrupts: int
    soft_interrupts: int
    syscalls: int

class scpufreq(NamedTuple):
    current: float
    min: float
    max: float

class shwtemp(NamedTuple):
    label: str
    current: float
    high: float | None
    critical: float | None

class sbattery(NamedTuple):
    percent: int
    secsleft: int
    power_plugged: bool

class sfan(NamedTuple):
    label: str
    current: int

class pcputimes(NamedTuple):
    user: float
    system: float
    children_user: float
    children_system: float

class popenfile(NamedTuple):
    path: str
    fd: int

class pthread(NamedTuple):
    id: int
    user_time: float
    system_time: float

class puids(NamedTuple):
    real: int
    effective: int
    saved: int

class pgids(NamedTuple):
    real: int
    effective: int
    saved: int

class pio(NamedTuple):
    read_count: int
    write_count: int
    read_bytes: int
    write_bytes: int

class pionice(NamedTuple):
    ioclass: int
    value: int

class pctxsw(NamedTuple):
    voluntary: int
    involuntary: int

class pconn(NamedTuple):
    fd: int
    family: AddressFamily
    type: SocketKind
    laddr: addr
    raddr: addr
    status: str

class addr(NamedTuple):
    ip: str
    port: int

conn_tmap: dict[str, tuple[list[AddressFamily], list[SocketKind]]]

class Error(Exception):
    msg: str
    def __init__(self, msg: str = ...) -> None: ...

class NoSuchProcess(Error):
    pid: int
    name: str | None
    msg: str
    def __init__(self, pid: int, name: str | None = None, msg: str | None = None) -> None: ...

class ZombieProcess(NoSuchProcess):
    ppid: int | None
    def __init__(self, pid: int, name: str | None = None, ppid: int | None = None, msg: str | None = None) -> None: ...

class AccessDenied(Error):
    pid: int | None
    name: str | None
    msg: str
    def __init__(self, pid: int | None = None, name: str | None = None, msg: str | None = None) -> None: ...

class TimeoutExpired(Error):
    seconds: float
    pid: int | None
    name: str | None
    msg: str
    def __init__(self, seconds: float, pid: int | None = None, name: str | None = None) -> None: ...

_P = ParamSpec("_P")
_R = TypeVar("_R")

def usage_percent(used: ConvertibleToFloat, total: float, round_: SupportsIndex | None = None) -> float: ...

# returned function has `cache_clear()` attribute:
def memoize(fun: Callable[_P, _R]) -> Callable[_P, _R]: ...

# returned function has `cache_activate(proc)` and `cache_deactivate(proc)` attributes:
def memoize_when_activated(fun: Callable[_P, _R]) -> Callable[_P, _R]: ...
def isfile_strict(path: StrOrBytesPath) -> bool: ...
def path_exists_strict(path: StrOrBytesPath) -> bool: ...
def supports_ipv6() -> bool: ...
def parse_environ_block(data: str) -> dict[str, str]: ...
def sockfam_to_enum(num: int) -> AddressFamily: ...
def socktype_to_enum(num: int) -> SocketKind: ...
@overload
def conn_to_ntuple(fd: int, fam: int, type_: int, laddr, raddr, status: str, status_map, pid: int) -> sconn: ...
@overload
def conn_to_ntuple(fd: int, fam: int, type_: int, laddr, raddr, status: str, status_map, pid: None = None) -> pconn: ...
def deprecated_method(replacement: str) -> Callable[[Callable[_P, _R]], Callable[_P, _R]]: ...

class _WrapNumbers:
    lock: threading.Lock
    cache: dict[Incomplete, Incomplete]
    reminders: dict[Incomplete, Incomplete]
    reminder_keys: dict[Incomplete, Incomplete]
    def __init__(self) -> None: ...
    def run(self, input_dict, name): ...
    def cache_clear(self, name=None) -> None: ...
    def cache_info(self) -> tuple[dict[Incomplete, Incomplete], dict[Incomplete, Incomplete], dict[Incomplete, Incomplete]]: ...

def wrap_numbers(input_dict, name: str): ...
def open_binary(fname: FileDescriptorOrPath) -> BinaryIO: ...
def open_text(fname: FileDescriptorOrPath) -> io.TextIOWrapper: ...
def cat(fname: FileDescriptorOrPath, fallback=..., _open=...): ...
def bcat(fname: FileDescriptorOrPath, fallback=...): ...
def bytes2human(n: int, format: str = "%(value).1f%(symbol)s") -> str: ...
def get_procfs_path() -> str: ...
def decode(s: bytes) -> str: ...
def term_supports_colors(file: SupportsWrite[str] = ...) -> bool: ...
def hilite(s: str, color: str | None = None, bold: bool = False) -> str: ...
def print_color(s: str, color: str | None = None, bold: bool = False, file: SupportsWrite[str] = ...) -> None: ...
def debug(msg: str | Exception) -> None: ...

__all__ = [
    # OS constants
    "FREEBSD",
    "BSD",
    "LINUX",
    "NETBSD",
    "OPENBSD",
    "MACOS",
    "OSX",
    "POSIX",
    "SUNOS",
    "WINDOWS",
    # connection constants
    "CONN_CLOSE",
    "CONN_CLOSE_WAIT",
    "CONN_CLOSING",
    "CONN_ESTABLISHED",
    "CONN_FIN_WAIT1",
    "CONN_FIN_WAIT2",
    "CONN_LAST_ACK",
    "CONN_LISTEN",
    "CONN_NONE",
    "CONN_SYN_RECV",
    "CONN_SYN_SENT",
    "CONN_TIME_WAIT",
    # net constants
    "NIC_DUPLEX_FULL",
    "NIC_DUPLEX_HALF",
    "NIC_DUPLEX_UNKNOWN",
    # process status constants
    "STATUS_DEAD",
    "STATUS_DISK_SLEEP",
    "STATUS_IDLE",
    "STATUS_LOCKED",
    "STATUS_RUNNING",
    "STATUS_SLEEPING",
    "STATUS_STOPPED",
    "STATUS_SUSPENDED",
    "STATUS_TRACING_STOP",
    "STATUS_WAITING",
    "STATUS_WAKE_KILL",
    "STATUS_WAKING",
    "STATUS_ZOMBIE",
    "STATUS_PARKED",
    # other constants
    "ENCODING",
    "ENCODING_ERRS",
    "AF_INET6",
    # named tuples
    "pconn",
    "pcputimes",
    "pctxsw",
    "pgids",
    "pio",
    "pionice",
    "popenfile",
    "pthread",
    "puids",
    "sconn",
    "scpustats",
    "sdiskio",
    "sdiskpart",
    "sdiskusage",
    "snetio",
    "snicaddr",
    "snicstats",
    "sswap",
    "suser",
    # utility functions
    "conn_tmap",
    "deprecated_method",
    "isfile_strict",
    "memoize",
    "parse_environ_block",
    "path_exists_strict",
    "usage_percent",
    "supports_ipv6",
    "sockfam_to_enum",
    "socktype_to_enum",
    "wrap_numbers",
    "open_text",
    "open_binary",
    "cat",
    "bcat",
    "bytes2human",
    "conn_to_ntuple",
    "debug",
    # shell utils
    "hilite",
    "term_supports_colors",
    "print_color",
]



from _typeshed import Incomplete
from collections.abc import Callable
from types import ModuleType

def getDomBuilder(DomImplementation) -> dict[str, Incomplete]: ...

getDomModule: Callable[..., ModuleType]



from _typeshed import Incomplete, SupportsItems, SupportsKeysAndGetItem, Unused
from collections.abc import Generator, Hashable, Iterable, Iterator
from typing import Literal, TypeVar, overload

from networkx import _dispatchable
from networkx.algorithms.planarity import PlanarEmbedding
from networkx.classes.digraph import DiGraph
from networkx.classes.graph import Graph, _NBunch, _Node
from networkx.classes.multigraph import MultiGraph

__all__ = [
    "nodes",
    "edges",
    "degree",
    "degree_histogram",
    "neighbors",
    "number_of_nodes",
    "number_of_edges",
    "density",
    "is_directed",
    "freeze",
    "is_frozen",
    "subgraph",
    "induced_subgraph",
    "edge_subgraph",
    "restricted_view",
    "to_directed",
    "to_undirected",
    "add_star",
    "add_path",
    "add_cycle",
    "create_empty_copy",
    "set_node_attributes",
    "get_node_attributes",
    "remove_node_attributes",
    "set_edge_attributes",
    "get_edge_attributes",
    "remove_edge_attributes",
    "all_neighbors",
    "non_neighbors",
    "non_edges",
    "common_neighbors",
    "is_weighted",
    "is_negatively_weighted",
    "is_empty",
    "selfloop_edges",
    "nodes_with_selfloops",
    "number_of_selfloops",
    "path_weight",
    "is_path",
]

_U = TypeVar("_U")

def nodes(G: Graph[_Node]): ...
def edges(G: Graph[_Node], nbunch=None): ...
def degree(G: Graph[_Node], nbunch=None, weight=None): ...
def neighbors(G: Graph[_Node], n): ...
def number_of_nodes(G: Graph[_Node]): ...
def number_of_edges(G: Graph[_Node]): ...
def density(G: Graph[_Node]): ...
def degree_histogram(G: Graph[_Node]) -> list[int]: ...
@overload
def is_directed(G: PlanarEmbedding[Hashable]) -> Literal[False]: ...  # type: ignore[misc] # Incompatible return types
@overload
def is_directed(G: DiGraph[Hashable]) -> Literal[True]: ...  # type: ignore[misc] # Incompatible return types
@overload
def is_directed(G: Graph[Hashable]) -> Literal[False]: ...
def freeze(G: Graph[_Node]): ...
def is_frozen(G: Graph[Incomplete]) -> bool: ...
def add_star(G_to_add_to, nodes_for_star, **attr) -> None: ...
def add_path(G_to_add_to, nodes_for_path, **attr) -> None: ...
def add_cycle(G_to_add_to, nodes_for_cycle, **attr) -> None: ...
def subgraph(G: Graph[_Node], nbunch): ...
def induced_subgraph(G: Graph[_Node], nbunch: _NBunch[_Node]) -> Graph[_Node]: ...
def edge_subgraph(G: Graph[_Node], edges): ...
def restricted_view(G: Graph[_Node], nodes, edges): ...
def to_directed(graph): ...
def to_undirected(graph): ...
def create_empty_copy(G: Graph[_Node], with_data: bool = True): ...

# incomplete: Can "Any scalar value" be enforced?
@overload
def set_node_attributes(
    G: Graph[Hashable],
    values: SupportsItems[_Node, Unused],
    name: str,
    *,
    backend=None,  # @_dispatchable adds these arguments, but we can't use this decorator with @overload
    **backend_kwargs,
) -> None: ...
@overload
def set_node_attributes(
    G: Graph[_Node],
    values: SupportsItems[_Node, SupportsKeysAndGetItem[Incomplete, Incomplete] | Iterable[tuple[Incomplete, Incomplete]]],
    name: None = None,
    *,
    backend=None,
    **backend_kwargs,
) -> None: ...
@_dispatchable
def get_node_attributes(G: Graph[_Node], name: str, default=None) -> dict[_Node, Incomplete]: ...
@_dispatchable
def remove_node_attributes(G: Graph[_Node], *attr_names, nbunch=None) -> None: ...
@overload
def set_edge_attributes(
    G: Graph[_Node],
    values: SupportsItems[tuple[_Node, _Node], Incomplete],
    name: str,
    *,
    backend: str | None = None,  # @_dispatchable adds these arguments, but we can't use this decorator with @overload
    **backend_kwargs,
) -> None: ...
@overload
def set_edge_attributes(
    G: MultiGraph[_Node],
    values: dict[tuple[_Node, _Node, Incomplete], Incomplete],
    name: str,
    *,
    backend: str | None = None,
    **backend_kwargs,
) -> None: ...
@overload
def set_edge_attributes(
    G: Graph[Hashable], values, name: None = None, *, backend: str | None = None, **backend_kwargs
) -> None: ...
@_dispatchable
def get_edge_attributes(G: Graph[_Node], name: str, default=None) -> dict[tuple[_Node, _Node], Incomplete]: ...
@_dispatchable
def remove_edge_attributes(G: Graph[_Node], *attr_names, ebunch=None) -> None: ...
def all_neighbors(graph: Graph[_Node], node: _Node) -> Iterator[_Node]: ...
def non_neighbors(graph: Graph[_Node], node: _Node) -> Generator[_Node, None, None]: ...
def non_edges(graph: Graph[_Node]) -> Generator[tuple[_Node, _Node], None, None]: ...
def common_neighbors(G: Graph[_Node], u: _Node, v: _Node) -> Generator[_Node, None, None]: ...
@_dispatchable
def is_weighted(G: Graph[_Node], edge: tuple[_Node, _Node] | None = None, weight: str = "weight") -> bool: ...
@_dispatchable
def is_negatively_weighted(G: Graph[_Node], edge: tuple[_Node, _Node] | None = None, weight: str = "weight") -> bool: ...
@_dispatchable
def is_empty(G: Graph[Hashable]) -> bool: ...
def nodes_with_selfloops(G: Graph[_Node]) -> Generator[_Node, None, None]: ...
@overload
def selfloop_edges(
    G: Graph[_Node], data: Literal[False] = False, keys: Literal[False] = False, default=None
) -> Generator[tuple[_Node, _Node], None, None]: ...
@overload
def selfloop_edges(
    G: Graph[_Node], data: Literal[True], keys: Literal[False] = False, default=None
) -> Generator[tuple[_Node, _Node, dict[str, Incomplete]], None, None]: ...
@overload
def selfloop_edges(
    G: Graph[_Node], data: str, keys: Literal[False] = False, default: _U | None = None
) -> Generator[tuple[_Node, _Node, _U], None, None]: ...
@overload
def selfloop_edges(
    G: Graph[_Node], data: Literal[False], keys: Literal[True], default=None
) -> Generator[tuple[_Node, _Node, int], None, None]: ...
@overload
def selfloop_edges(
    G: Graph[_Node], data: Literal[False] = False, *, keys: Literal[True], default=None
) -> Generator[tuple[_Node, _Node, int], None, None]: ...
@overload
def selfloop_edges(
    G: Graph[_Node], data: Literal[True], keys: Literal[True], default=None
) -> Generator[tuple[_Node, _Node, int, dict[str, Incomplete]], None, None]: ...
@overload
def selfloop_edges(
    G: Graph[_Node], data: str, keys: Literal[True], default: _U | None = None
) -> Generator[tuple[_Node, _Node, int, _U], None, None]: ...
@_dispatchable
def number_of_selfloops(G: Graph[Hashable]) -> int: ...
def is_path(G: Graph[_Node], path: Iterable[Incomplete]) -> bool: ...
def path_weight(G: Graph[_Node], path, weight) -> int: ...



from collections.abc import Callable
from logging import Logger
from typing import Any

import click

from ._cmd import ClickCmd

logger: Logger

def get_invoke(command: click.Command) -> Callable[[ClickCmd, str], bool]: ...
def get_help(command: click.Command) -> Callable[[ClickCmd], None]: ...
def get_complete(command: click.Command) -> Callable[[ClickCmd, str, str, int, int], list[str]]: ...

class ClickShell(ClickCmd):
    def add_command(self, cmd: click.Command, name: str) -> None: ...

def make_click_shell(
    ctx: click.Context,
    prompt: str | Callable[[], str] | Callable[[click.Context, str], str] | None = None,
    intro: str | None = None,
    hist_file: str | None = None,
) -> ClickShell: ...

class Shell(click.Group):
    def __init__(
        self,
        prompt: str | Callable[[], str] | Callable[[click.Context, str], str] | None = None,
        intro: str | None = None,
        hist_file: str | None = None,
        on_finished: Callable[[click.Context], None] | None = None,
        **attrs: Any,
    ) -> None: ...
    def add_command(self, cmd: click.Command, name: str | None = None) -> None: ...
    def invoke(self, ctx: click.Context) -> Any: ...



from typing import ClassVar

class AMQPObject:
    NAME: ClassVar[str]
    INDEX: ClassVar[int | None]
    def __eq__(self, other: AMQPObject | None) -> bool: ...  # type: ignore[override]

class Class(AMQPObject): ...

class Method(AMQPObject):
    # This is a class attribute in the implementation, but subclasses use @property,
    # so it's more convenient to use that here as well.
    @property
    def synchronous(self) -> bool: ...
    def get_properties(self) -> Properties: ...
    def get_body(self) -> str: ...

class Properties(AMQPObject): ...



from _typeshed import Incomplete
from collections.abc import Iterable, Mapping
from typing import ClassVar, final

from google._upb._message import Descriptor, FileDescriptor, MessageMeta
from google.protobuf import any_pb2, duration_pb2, message, timestamp_pb2, wrappers_pb2
from google.protobuf.internal import containers

DESCRIPTOR: FileDescriptor

@final
class Channel(message.Message, metaclass=MessageMeta):
    REF_FIELD_NUMBER: ClassVar[int]
    DATA_FIELD_NUMBER: ClassVar[int]
    CHANNEL_REF_FIELD_NUMBER: ClassVar[int]
    SUBCHANNEL_REF_FIELD_NUMBER: ClassVar[int]
    SOCKET_REF_FIELD_NUMBER: ClassVar[int]
    ref: ChannelRef
    data: ChannelData
    channel_ref: containers.RepeatedCompositeFieldContainer[ChannelRef]
    subchannel_ref: containers.RepeatedCompositeFieldContainer[SubchannelRef]
    socket_ref: containers.RepeatedCompositeFieldContainer[SocketRef]
    def __init__(
        self,
        ref: ChannelRef | Mapping[Incomplete, Incomplete] | None = ...,
        data: ChannelData | Mapping[Incomplete, Incomplete] | None = ...,
        channel_ref: Iterable[ChannelRef | Mapping[Incomplete, Incomplete]] | None = ...,
        subchannel_ref: Iterable[SubchannelRef | Mapping[Incomplete, Incomplete]] | None = ...,
        socket_ref: Iterable[SocketRef | Mapping[Incomplete, Incomplete]] | None = ...,
    ) -> None: ...
    DESCRIPTOR: Descriptor

@final
class Subchannel(message.Message, metaclass=MessageMeta):
    REF_FIELD_NUMBER: ClassVar[int]
    DATA_FIELD_NUMBER: ClassVar[int]
    CHANNEL_REF_FIELD_NUMBER: ClassVar[int]
    SUBCHANNEL_REF_FIELD_NUMBER: ClassVar[int]
    SOCKET_REF_FIELD_NUMBER: ClassVar[int]
    ref: SubchannelRef
    data: ChannelData
    channel_ref: containers.RepeatedCompositeFieldContainer[ChannelRef]
    subchannel_ref: containers.RepeatedCompositeFieldContainer[SubchannelRef]
    socket_ref: containers.RepeatedCompositeFieldContainer[SocketRef]
    def __init__(
        self,
        ref: SubchannelRef | Mapping[Incomplete, Incomplete] | None = ...,
        data: ChannelData | Mapping[Incomplete, Incomplete] | None = ...,
        channel_ref: Iterable[ChannelRef | Mapping[Incomplete, Incomplete]] | None = ...,
        subchannel_ref: Iterable[SubchannelRef | Mapping[Incomplete, Incomplete]] | None = ...,
        socket_ref: Iterable[SocketRef | Mapping[Incomplete, Incomplete]] | None = ...,
    ) -> None: ...
    DESCRIPTOR: Descriptor

@final
class ChannelConnectivityState(message.Message, metaclass=MessageMeta):
    State: Incomplete
    UNKNOWN: Incomplete
    IDLE: Incomplete
    CONNECTING: Incomplete
    READY: Incomplete
    TRANSIENT_FAILURE: Incomplete
    SHUTDOWN: Incomplete
    STATE_FIELD_NUMBER: ClassVar[int]
    state: Incomplete
    def __init__(self, state: Incomplete | str | None = ...) -> None: ...
    DESCRIPTOR: Descriptor

@final
class ChannelData(message.Message, metaclass=MessageMeta):
    STATE_FIELD_NUMBER: ClassVar[int]
    TARGET_FIELD_NUMBER: ClassVar[int]
    TRACE_FIELD_NUMBER: ClassVar[int]
    CALLS_STARTED_FIELD_NUMBER: ClassVar[int]
    CALLS_SUCCEEDED_FIELD_NUMBER: ClassVar[int]
    CALLS_FAILED_FIELD_NUMBER: ClassVar[int]
    LAST_CALL_STARTED_TIMESTAMP_FIELD_NUMBER: ClassVar[int]
    state: ChannelConnectivityState
    target: str
    trace: ChannelTrace
    calls_started: int
    calls_succeeded: int
    calls_failed: int
    last_call_started_timestamp: timestamp_pb2.Timestamp
    def __init__(
        self,
        state: ChannelConnectivityState | Mapping[Incomplete, Incomplete] | None = ...,
        target: str | None = ...,
        trace: ChannelTrace | Mapping[Incomplete, Incomplete] | None = ...,
        calls_started: int | None = ...,
        calls_succeeded: int | None = ...,
        calls_failed: int | None = ...,
        last_call_started_timestamp: timestamp_pb2.Timestamp | Mapping[Incomplete, Incomplete] | None = ...,
    ) -> None: ...
    DESCRIPTOR: Descriptor

@final
class ChannelTraceEvent(message.Message, metaclass=MessageMeta):
    Severity: Incomplete
    CT_UNKNOWN: Incomplete
    CT_INFO: Incomplete
    CT_WARNING: Incomplete
    CT_ERROR: Incomplete
    DESCRIPTION_FIELD_NUMBER: ClassVar[int]
    SEVERITY_FIELD_NUMBER: ClassVar[int]
    TIMESTAMP_FIELD_NUMBER: ClassVar[int]
    CHANNEL_REF_FIELD_NUMBER: ClassVar[int]
    SUBCHANNEL_REF_FIELD_NUMBER: ClassVar[int]
    description: str
    severity: Incomplete
    timestamp: timestamp_pb2.Timestamp
    channel_ref: ChannelRef
    subchannel_ref: SubchannelRef
    def __init__(
        self,
        description: str | None = ...,
        severity: Incomplete | str | None = ...,
        timestamp: timestamp_pb2.Timestamp | Mapping[Incomplete, Incomplete] | None = ...,
        channel_ref: ChannelRef | Mapping[Incomplete, Incomplete] | None = ...,
        subchannel_ref: SubchannelRef | Mapping[Incomplete, Incomplete] | None = ...,
    ) -> None: ...
    DESCRIPTOR: Descriptor

@final
class ChannelTrace(message.Message, metaclass=MessageMeta):
    NUM_EVENTS_LOGGED_FIELD_NUMBER: ClassVar[int]
    CREATION_TIMESTAMP_FIELD_NUMBER: ClassVar[int]
    EVENTS_FIELD_NUMBER: ClassVar[int]
    num_events_logged: int
    creation_timestamp: timestamp_pb2.Timestamp
    events: containers.RepeatedCompositeFieldContainer[ChannelTraceEvent]
    def __init__(
        self,
        num_events_logged: int | None = ...,
        creation_timestamp: timestamp_pb2.Timestamp | Mapping[Incomplete, Incomplete] | None = ...,
        events: Iterable[ChannelTraceEvent | Mapping[Incomplete, Incomplete]] | None = ...,
    ) -> None: ...
    DESCRIPTOR: Descriptor

@final
class ChannelRef(message.Message, metaclass=MessageMeta):
    CHANNEL_ID_FIELD_NUMBER: ClassVar[int]
    NAME_FIELD_NUMBER: ClassVar[int]
    channel_id: int
    name: str
    def __init__(self, channel_id: int | None = ..., name: str | None = ...) -> None: ...
    DESCRIPTOR: Descriptor

@final
class SubchannelRef(message.Message, metaclass=MessageMeta):
    SUBCHANNEL_ID_FIELD_NUMBER: ClassVar[int]
    NAME_FIELD_NUMBER: ClassVar[int]
    subchannel_id: int
    name: str
    def __init__(self, subchannel_id: int | None = ..., name: str | None = ...) -> None: ...
    DESCRIPTOR: Descriptor

@final
class SocketRef(message.Message, metaclass=MessageMeta):
    SOCKET_ID_FIELD_NUMBER: ClassVar[int]
    NAME_FIELD_NUMBER: ClassVar[int]
    socket_id: int
    name: str
    def __init__(self, socket_id: int | None = ..., name: str | None = ...) -> None: ...
    DESCRIPTOR: Descriptor

@final
class ServerRef(message.Message, metaclass=MessageMeta):
    SERVER_ID_FIELD_NUMBER: ClassVar[int]
    NAME_FIELD_NUMBER: ClassVar[int]
    server_id: int
    name: str
    def __init__(self, server_id: int | None = ..., name: str | None = ...) -> None: ...
    DESCRIPTOR: Descriptor

@final
class Server(message.Message, metaclass=MessageMeta):
    REF_FIELD_NUMBER: ClassVar[int]
    DATA_FIELD_NUMBER: ClassVar[int]
    LISTEN_SOCKET_FIELD_NUMBER: ClassVar[int]
    ref: ServerRef
    data: ServerData
    listen_socket: containers.RepeatedCompositeFieldContainer[SocketRef]
    def __init__(
        self,
        ref: ServerRef | Mapping[Incomplete, Incomplete] | None = ...,
        data: ServerData | Mapping[Incomplete, Incomplete] | None = ...,
        listen_socket: Iterable[SocketRef | Mapping[Incomplete, Incomplete]] | None = ...,
    ) -> None: ...
    DESCRIPTOR: Descriptor

@final
class ServerData(message.Message, metaclass=MessageMeta):
    TRACE_FIELD_NUMBER: ClassVar[int]
    CALLS_STARTED_FIELD_NUMBER: ClassVar[int]
    CALLS_SUCCEEDED_FIELD_NUMBER: ClassVar[int]
    CALLS_FAILED_FIELD_NUMBER: ClassVar[int]
    LAST_CALL_STARTED_TIMESTAMP_FIELD_NUMBER: ClassVar[int]
    trace: ChannelTrace
    calls_started: int
    calls_succeeded: int
    calls_failed: int
    last_call_started_timestamp: timestamp_pb2.Timestamp
    def __init__(
        self,
        trace: ChannelTrace | Mapping[Incomplete, Incomplete] | None = ...,
        calls_started: int | None = ...,
        calls_succeeded: int | None = ...,
        calls_failed: int | None = ...,
        last_call_started_timestamp: timestamp_pb2.Timestamp | Mapping[Incomplete, Incomplete] | None = ...,
    ) -> None: ...
    DESCRIPTOR: Descriptor

@final
class Socket(message.Message, metaclass=MessageMeta):
    REF_FIELD_NUMBER: ClassVar[int]
    DATA_FIELD_NUMBER: ClassVar[int]
    LOCAL_FIELD_NUMBER: ClassVar[int]
    REMOTE_FIELD_NUMBER: ClassVar[int]
    SECURITY_FIELD_NUMBER: ClassVar[int]
    REMOTE_NAME_FIELD_NUMBER: ClassVar[int]
    ref: SocketRef
    data: SocketData
    local: Address
    remote: Address
    security: Security
    remote_name: str
    def __init__(
        self,
        ref: SocketRef | Mapping[Incomplete, Incomplete] | None = ...,
        data: SocketData | Mapping[Incomplete, Incomplete] | None = ...,
        local: Address | Mapping[Incomplete, Incomplete] | None = ...,
        remote: Address | Mapping[Incomplete, Incomplete] | None = ...,
        security: Security | Mapping[Incomplete, Incomplete] | None = ...,
        remote_name: str | None = ...,
    ) -> None: ...
    DESCRIPTOR: Descriptor

@final
class SocketData(message.Message, metaclass=MessageMeta):
    STREAMS_STARTED_FIELD_NUMBER: ClassVar[int]
    STREAMS_SUCCEEDED_FIELD_NUMBER: ClassVar[int]
    STREAMS_FAILED_FIELD_NUMBER: ClassVar[int]
    MESSAGES_SENT_FIELD_NUMBER: ClassVar[int]
    MESSAGES_RECEIVED_FIELD_NUMBER: ClassVar[int]
    KEEP_ALIVES_SENT_FIELD_NUMBER: ClassVar[int]
    LAST_LOCAL_STREAM_CREATED_TIMESTAMP_FIELD_NUMBER: ClassVar[int]
    LAST_REMOTE_STREAM_CREATED_TIMESTAMP_FIELD_NUMBER: ClassVar[int]
    LAST_MESSAGE_SENT_TIMESTAMP_FIELD_NUMBER: ClassVar[int]
    LAST_MESSAGE_RECEIVED_TIMESTAMP_FIELD_NUMBER: ClassVar[int]
    LOCAL_FLOW_CONTROL_WINDOW_FIELD_NUMBER: ClassVar[int]
    REMOTE_FLOW_CONTROL_WINDOW_FIELD_NUMBER: ClassVar[int]
    OPTION_FIELD_NUMBER: ClassVar[int]
    streams_started: int
    streams_succeeded: int
    streams_failed: int
    messages_sent: int
    messages_received: int
    keep_alives_sent: int
    last_local_stream_created_timestamp: timestamp_pb2.Timestamp
    last_remote_stream_created_timestamp: timestamp_pb2.Timestamp
    last_message_sent_timestamp: timestamp_pb2.Timestamp
    last_message_received_timestamp: timestamp_pb2.Timestamp
    local_flow_control_window: wrappers_pb2.Int64Value
    remote_flow_control_window: wrappers_pb2.Int64Value
    option: containers.RepeatedCompositeFieldContainer[SocketOption]
    def __init__(
        self,
        streams_started: int | None = ...,
        streams_succeeded: int | None = ...,
        streams_failed: int | None = ...,
        messages_sent: int | None = ...,
        messages_received: int | None = ...,
        keep_alives_sent: int | None = ...,
        last_local_stream_created_timestamp: timestamp_pb2.Timestamp | Mapping[Incomplete, Incomplete] | None = ...,
        last_remote_stream_created_timestamp: timestamp_pb2.Timestamp | Mapping[Incomplete, Incomplete] | None = ...,
        last_message_sent_timestamp: timestamp_pb2.Timestamp | Mapping[Incomplete, Incomplete] | None = ...,
        last_message_received_timestamp: timestamp_pb2.Timestamp | Mapping[Incomplete, Incomplete] | None = ...,
        local_flow_control_window: wrappers_pb2.Int64Value | Mapping[Incomplete, Incomplete] | None = ...,
        remote_flow_control_window: wrappers_pb2.Int64Value | Mapping[Incomplete, Incomplete] | None = ...,
        option: Iterable[SocketOption | Mapping[Incomplete, Incomplete]] | None = ...,
    ) -> None: ...
    DESCRIPTOR: Descriptor

@final
class Address(message.Message, metaclass=MessageMeta):
    @final
    class TcpIpAddress(message.Message, metaclass=MessageMeta):
        IP_ADDRESS_FIELD_NUMBER: ClassVar[int]
        PORT_FIELD_NUMBER: ClassVar[int]
        ip_address: bytes
        port: int
        def __init__(self, ip_address: bytes | None = ..., port: int | None = ...) -> None: ...

    @final
    class UdsAddress(message.Message, metaclass=MessageMeta):
        FILENAME_FIELD_NUMBER: ClassVar[int]
        filename: str
        def __init__(self, filename: str | None = ...) -> None: ...

    @final
    class OtherAddress(message.Message, metaclass=MessageMeta):
        NAME_FIELD_NUMBER: ClassVar[int]
        VALUE_FIELD_NUMBER: ClassVar[int]
        name: str
        value: any_pb2.Any
        def __init__(self, name: str | None = ..., value: any_pb2.Any | Mapping[Incomplete, Incomplete] | None = ...) -> None: ...

    TCPIP_ADDRESS_FIELD_NUMBER: ClassVar[int]
    UDS_ADDRESS_FIELD_NUMBER: ClassVar[int]
    OTHER_ADDRESS_FIELD_NUMBER: ClassVar[int]
    tcpip_address: Address.TcpIpAddress
    uds_address: Address.UdsAddress
    other_address: Address.OtherAddress
    def __init__(
        self,
        tcpip_address: Address.TcpIpAddress | Mapping[Incomplete, Incomplete] | None = ...,
        uds_address: Address.UdsAddress | Mapping[Incomplete, Incomplete] | None = ...,
        other_address: Address.OtherAddress | Mapping[Incomplete, Incomplete] | None = ...,
    ) -> None: ...
    DESCRIPTOR: Descriptor

@final
class Security(message.Message, metaclass=MessageMeta):
    @final
    class Tls(message.Message, metaclass=MessageMeta):
        STANDARD_NAME_FIELD_NUMBER: ClassVar[int]
        OTHER_NAME_FIELD_NUMBER: ClassVar[int]
        LOCAL_CERTIFICATE_FIELD_NUMBER: ClassVar[int]
        REMOTE_CERTIFICATE_FIELD_NUMBER: ClassVar[int]
        standard_name: str
        other_name: str
        local_certificate: bytes
        remote_certificate: bytes
        def __init__(
            self,
            standard_name: str | None = ...,
            other_name: str | None = ...,
            local_certificate: bytes | None = ...,
            remote_certificate: bytes | None = ...,
        ) -> None: ...

    @final
    class OtherSecurity(message.Message, metaclass=MessageMeta):
        NAME_FIELD_NUMBER: ClassVar[int]
        VALUE_FIELD_NUMBER: ClassVar[int]
        name: str
        value: any_pb2.Any
        def __init__(self, name: str | None = ..., value: any_pb2.Any | Mapping[Incomplete, Incomplete] | None = ...) -> None: ...

    TLS_FIELD_NUMBER: ClassVar[int]
    OTHER_FIELD_NUMBER: ClassVar[int]
    tls: Security.Tls
    other: Security.OtherSecurity
    def __init__(
        self,
        tls: Security.Tls | Mapping[Incomplete, Incomplete] | None = ...,
        other: Security.OtherSecurity | Mapping[Incomplete, Incomplete] | None = ...,
    ) -> None: ...
    DESCRIPTOR: Descriptor

@final
class SocketOption(message.Message, metaclass=MessageMeta):
    NAME_FIELD_NUMBER: ClassVar[int]
    VALUE_FIELD_NUMBER: ClassVar[int]
    ADDITIONAL_FIELD_NUMBER: ClassVar[int]
    name: str
    value: str
    additional: any_pb2.Any
    def __init__(
        self,
        name: str | None = ...,
        value: str | None = ...,
        additional: any_pb2.Any | Mapping[Incomplete, Incomplete] | None = ...,
    ) -> None: ...
    DESCRIPTOR: Descriptor

@final
class SocketOptionTimeout(message.Message, metaclass=MessageMeta):
    DURATION_FIELD_NUMBER: ClassVar[int]
    duration: duration_pb2.Duration
    def __init__(self, duration: duration_pb2.Duration | Mapping[Incomplete, Incomplete] | None = ...) -> None: ...
    DESCRIPTOR: Descriptor

@final
class SocketOptionLinger(message.Message, metaclass=MessageMeta):
    ACTIVE_FIELD_NUMBER: ClassVar[int]
    DURATION_FIELD_NUMBER: ClassVar[int]
    active: bool
    duration: duration_pb2.Duration
    def __init__(
        self, active: bool = ..., duration: duration_pb2.Duration | Mapping[Incomplete, Incomplete] | None = ...
    ) -> None: ...
    DESCRIPTOR: Descriptor

@final
class SocketOptionTcpInfo(message.Message, metaclass=MessageMeta):
    TCPI_STATE_FIELD_NUMBER: ClassVar[int]
    TCPI_CA_STATE_FIELD_NUMBER: ClassVar[int]
    TCPI_RETRANSMITS_FIELD_NUMBER: ClassVar[int]
    TCPI_PROBES_FIELD_NUMBER: ClassVar[int]
    TCPI_BACKOFF_FIELD_NUMBER: ClassVar[int]
    TCPI_OPTIONS_FIELD_NUMBER: ClassVar[int]
    TCPI_SND_WSCALE_FIELD_NUMBER: ClassVar[int]
    TCPI_RCV_WSCALE_FIELD_NUMBER: ClassVar[int]
    TCPI_RTO_FIELD_NUMBER: ClassVar[int]
    TCPI_ATO_FIELD_NUMBER: ClassVar[int]
    TCPI_SND_MSS_FIELD_NUMBER: ClassVar[int]
    TCPI_RCV_MSS_FIELD_NUMBER: ClassVar[int]
    TCPI_UNACKED_FIELD_NUMBER: ClassVar[int]
    TCPI_SACKED_FIELD_NUMBER: ClassVar[int]
    TCPI_LOST_FIELD_NUMBER: ClassVar[int]
    TCPI_RETRANS_FIELD_NUMBER: ClassVar[int]
    TCPI_FACKETS_FIELD_NUMBER: ClassVar[int]
    TCPI_LAST_DATA_SENT_FIELD_NUMBER: ClassVar[int]
    TCPI_LAST_ACK_SENT_FIELD_NUMBER: ClassVar[int]
    TCPI_LAST_DATA_RECV_FIELD_NUMBER: ClassVar[int]
    TCPI_LAST_ACK_RECV_FIELD_NUMBER: ClassVar[int]
    TCPI_PMTU_FIELD_NUMBER: ClassVar[int]
    TCPI_RCV_SSTHRESH_FIELD_NUMBER: ClassVar[int]
    TCPI_RTT_FIELD_NUMBER: ClassVar[int]
    TCPI_RTTVAR_FIELD_NUMBER: ClassVar[int]
    TCPI_SND_SSTHRESH_FIELD_NUMBER: ClassVar[int]
    TCPI_SND_CWND_FIELD_NUMBER: ClassVar[int]
    TCPI_ADVMSS_FIELD_NUMBER: ClassVar[int]
    TCPI_REORDERING_FIELD_NUMBER: ClassVar[int]
    tcpi_state: int
    tcpi_ca_state: int
    tcpi_retransmits: int
    tcpi_probes: int
    tcpi_backoff: int
    tcpi_options: int
    tcpi_snd_wscale: int
    tcpi_rcv_wscale: int
    tcpi_rto: int
    tcpi_ato: int
    tcpi_snd_mss: int
    tcpi_rcv_mss: int
    tcpi_unacked: int
    tcpi_sacked: int
    tcpi_lost: int
    tcpi_retrans: int
    tcpi_fackets: int
    tcpi_last_data_sent: int
    tcpi_last_ack_sent: int
    tcpi_last_data_recv: int
    tcpi_last_ack_recv: int
    tcpi_pmtu: int
    tcpi_rcv_ssthresh: int
    tcpi_rtt: int
    tcpi_rttvar: int
    tcpi_snd_ssthresh: int
    tcpi_snd_cwnd: int
    tcpi_advmss: int
    tcpi_reordering: int
    def __init__(
        self,
        tcpi_state: int | None = ...,
        tcpi_ca_state: int | None = ...,
        tcpi_retransmits: int | None = ...,
        tcpi_probes: int | None = ...,
        tcpi_backoff: int | None = ...,
        tcpi_options: int | None = ...,
        tcpi_snd_wscale: int | None = ...,
        tcpi_rcv_wscale: int | None = ...,
        tcpi_rto: int | None = ...,
        tcpi_ato: int | None = ...,
        tcpi_snd_mss: int | None = ...,
        tcpi_rcv_mss: int | None = ...,
        tcpi_unacked: int | None = ...,
        tcpi_sacked: int | None = ...,
        tcpi_lost: int | None = ...,
        tcpi_retrans: int | None = ...,
        tcpi_fackets: int | None = ...,
        tcpi_last_data_sent: int | None = ...,
        tcpi_last_ack_sent: int | None = ...,
        tcpi_last_data_recv: int | None = ...,
        tcpi_last_ack_recv: int | None = ...,
        tcpi_pmtu: int | None = ...,
        tcpi_rcv_ssthresh: int | None = ...,
        tcpi_rtt: int | None = ...,
        tcpi_rttvar: int | None = ...,
        tcpi_snd_ssthresh: int | None = ...,
        tcpi_snd_cwnd: int | None = ...,
        tcpi_advmss: int | None = ...,
        tcpi_reordering: int | None = ...,
    ) -> None: ...
    DESCRIPTOR: Descriptor

@final
class GetTopChannelsRequest(message.Message, metaclass=MessageMeta):
    START_CHANNEL_ID_FIELD_NUMBER: ClassVar[int]
    MAX_RESULTS_FIELD_NUMBER: ClassVar[int]
    start_channel_id: int
    max_results: int
    def __init__(self, start_channel_id: int | None = ..., max_results: int | None = ...) -> None: ...
    DESCRIPTOR: Descriptor

@final
class GetTopChannelsResponse(message.Message, metaclass=MessageMeta):
    CHANNEL_FIELD_NUMBER: ClassVar[int]
    END_FIELD_NUMBER: ClassVar[int]
    channel: containers.RepeatedCompositeFieldContainer[Channel]
    end: bool
    def __init__(self, channel: Iterable[Channel | Mapping[Incomplete, Incomplete]] | None = ..., end: bool = ...) -> None: ...
    DESCRIPTOR: Descriptor

@final
class GetServersRequest(message.Message, metaclass=MessageMeta):
    START_SERVER_ID_FIELD_NUMBER: ClassVar[int]
    MAX_RESULTS_FIELD_NUMBER: ClassVar[int]
    start_server_id: int
    max_results: int
    def __init__(self, start_server_id: int | None = ..., max_results: int | None = ...) -> None: ...
    DESCRIPTOR: Descriptor

@final
class GetServersResponse(message.Message, metaclass=MessageMeta):
    SERVER_FIELD_NUMBER: ClassVar[int]
    END_FIELD_NUMBER: ClassVar[int]
    server: containers.RepeatedCompositeFieldContainer[Server]
    end: bool
    def __init__(self, server: Iterable[Server | Mapping[Incomplete, Incomplete]] | None = ..., end: bool = ...) -> None: ...
    DESCRIPTOR: Descriptor

@final
class GetServerRequest(message.Message, metaclass=MessageMeta):
    SERVER_ID_FIELD_NUMBER: ClassVar[int]
    server_id: int
    def __init__(self, server_id: int | None = ...) -> None: ...
    DESCRIPTOR: Descriptor

@final
class GetServerResponse(message.Message, metaclass=MessageMeta):
    SERVER_FIELD_NUMBER: ClassVar[int]
    server: Server
    def __init__(self, server: Server | Mapping[Incomplete, Incomplete] | None = ...) -> None: ...
    DESCRIPTOR: Descriptor

@final
class GetServerSocketsRequest(message.Message, metaclass=MessageMeta):
    SERVER_ID_FIELD_NUMBER: ClassVar[int]
    START_SOCKET_ID_FIELD_NUMBER: ClassVar[int]
    MAX_RESULTS_FIELD_NUMBER: ClassVar[int]
    server_id: int
    start_socket_id: int
    max_results: int
    def __init__(self, server_id: int | None = ..., start_socket_id: int | None = ..., max_results: int | None = ...) -> None: ...
    DESCRIPTOR: Descriptor

@final
class GetServerSocketsResponse(message.Message, metaclass=MessageMeta):
    SOCKET_REF_FIELD_NUMBER: ClassVar[int]
    END_FIELD_NUMBER: ClassVar[int]
    socket_ref: containers.RepeatedCompositeFieldContainer[SocketRef]
    end: bool
    def __init__(
        self, socket_ref: Iterable[SocketRef | Mapping[Incomplete, Incomplete]] | None = ..., end: bool = ...
    ) -> None: ...
    DESCRIPTOR: Descriptor

@final
class GetChannelRequest(message.Message, metaclass=MessageMeta):
    CHANNEL_ID_FIELD_NUMBER: ClassVar[int]
    channel_id: int
    def __init__(self, channel_id: int | None = ...) -> None: ...
    DESCRIPTOR: Descriptor

@final
class GetChannelResponse(message.Message, metaclass=MessageMeta):
    CHANNEL_FIELD_NUMBER: ClassVar[int]
    channel: Channel
    def __init__(self, channel: Channel | Mapping[Incomplete, Incomplete] | None = ...) -> None: ...
    DESCRIPTOR: Descriptor

@final
class GetSubchannelRequest(message.Message, metaclass=MessageMeta):
    SUBCHANNEL_ID_FIELD_NUMBER: ClassVar[int]
    subchannel_id: int
    def __init__(self, subchannel_id: int | None = ...) -> None: ...
    DESCRIPTOR: Descriptor

@final
class GetSubchannelResponse(message.Message, metaclass=MessageMeta):
    SUBCHANNEL_FIELD_NUMBER: ClassVar[int]
    subchannel: Subchannel
    def __init__(self, subchannel: Subchannel | Mapping[Incomplete, Incomplete] | None = ...) -> None: ...
    DESCRIPTOR: Descriptor

@final
class GetSocketRequest(message.Message, metaclass=MessageMeta):
    SOCKET_ID_FIELD_NUMBER: ClassVar[int]
    SUMMARY_FIELD_NUMBER: ClassVar[int]
    socket_id: int
    summary: bool
    def __init__(self, socket_id: int | None = ..., summary: bool = ...) -> None: ...
    DESCRIPTOR: Descriptor

@final
class GetSocketResponse(message.Message, metaclass=MessageMeta):
    SOCKET_FIELD_NUMBER: ClassVar[int]
    socket: Socket
    def __init__(self, socket: Socket | Mapping[Incomplete, Incomplete] | None = ...) -> None: ...
    DESCRIPTOR: Descriptor



from _typeshed import Incomplete
from collections.abc import Generator

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["k_edge_components", "k_edge_subgraphs", "bridge_components", "EdgeComponentAuxGraph"]

@_dispatchable
def k_edge_components(G: Graph[_Node], k: int): ...
@_dispatchable
def k_edge_subgraphs(G: Graph[_Node], k: int): ...
@_dispatchable
def bridge_components(G: Graph[_Node]) -> Generator[Incomplete, Incomplete, None]: ...

class EdgeComponentAuxGraph:
    A: Incomplete
    H: Incomplete

    @classmethod
    def construct(cls, G: Graph[_Node]): ...
    def k_edge_components(self, k: int) -> Generator[Incomplete, Incomplete, None]: ...
    def k_edge_subgraphs(self, k: int) -> Generator[Incomplete, Incomplete, None]: ...

@_dispatchable
def general_k_edge_subgraphs(G: Graph[_Node], k): ...



from typing import Final

__version__: Final[str]

def getStateDelta(shape): ...

class StateTracker:
    def __init__(self, defaults=None, defaultObj=None) -> None: ...
    def push(self, delta) -> None: ...
    def pop(self): ...
    def getState(self): ...
    def getCTM(self): ...
    def __getitem__(self, key): ...
    def __setitem__(self, key, value) -> None: ...

def testStateTracker() -> None: ...
def renderScaledDrawing(d): ...

class Renderer:
    def undefined(self, operation) -> None: ...
    def draw(self, drawing, canvas, x: int = 0, y: int = 0, showBoundary=...) -> None: ...
    def initState(self, x, y) -> None: ...
    def pop(self) -> None: ...
    def drawNode(self, node) -> None: ...
    def getStateValue(self, key): ...
    def fillDerivedValues(self, node) -> None: ...
    def drawNodeDispatcher(self, anode) -> None: ...
    def drawGroup(self, group) -> None: ...
    def drawWedge(self, wedge) -> None: ...
    def drawPath(self, path) -> None: ...
    def drawRect(self, rect) -> None: ...
    def drawLine(self, line) -> None: ...
    def drawCircle(self, circle) -> None: ...
    def drawPolyLine(self, p) -> None: ...
    def drawEllipse(self, ellipse) -> None: ...
    def drawPolygon(self, p) -> None: ...
    def drawString(self, stringObj) -> None: ...
    def applyStateChanges(self, delta, newState) -> None: ...
    def drawImage(self, *args, **kwds) -> None: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import sys
import typing

import google.protobuf.any_pb2
import google.protobuf.descriptor
import google.protobuf.duration_pb2
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import google.protobuf.timestamp_pb2

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class PassMetrics(google.protobuf.message.Message):
    """Defines pass specific metrics."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MODULE_ID_FIELD_NUMBER: builtins.int
    PASS_NAME_FIELD_NUMBER: builtins.int
    PASS_DURATION_FIELD_NUMBER: builtins.int
    CUSTOM_METRICS_FIELD_NUMBER: builtins.int
    module_id: builtins.int
    """Unique ID of the module on which the pass was run."""
    pass_name: builtins.str
    """The name of the pass."""
    @property
    def pass_duration(self) -> google.protobuf.duration_pb2.Duration:
        """Duration of the pass."""

    @property
    def custom_metrics(self) -> google.protobuf.any_pb2.Any:
        """Custom pass metrics. This is kept opaque, via `google.protobuf.Any`, in
        order to decouple pass agnostic compilation logs from possibly proprietary
        compiler passes.
        """

    def __init__(
        self,
        *,
        module_id: builtins.int | None = ...,
        pass_name: builtins.str | None = ...,
        pass_duration: google.protobuf.duration_pb2.Duration | None = ...,
        custom_metrics: google.protobuf.any_pb2.Any | None = ...,
    ) -> None: ...
    def HasField(
        self, field_name: typing.Literal["custom_metrics", b"custom_metrics", "pass_duration", b"pass_duration"]
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "custom_metrics",
            b"custom_metrics",
            "module_id",
            b"module_id",
            "pass_duration",
            b"pass_duration",
            "pass_name",
            b"pass_name",
        ],
    ) -> None: ...

global___PassMetrics = PassMetrics

@typing.final
class JobInfo(google.protobuf.message.Message):
    """Defines compilation job information."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    CELL_FIELD_NUMBER: builtins.int
    USER_FIELD_NUMBER: builtins.int
    UID_FIELD_NUMBER: builtins.int
    TASK_ID_FIELD_NUMBER: builtins.int
    TASK_UID_FIELD_NUMBER: builtins.int
    name: builtins.str
    """Name of the job running compilation."""
    cell: builtins.str
    """Cell in which the job is running."""
    user: builtins.str
    """User running the job."""
    uid: builtins.int
    """Unique id when combined with user and cell field."""
    task_id: builtins.int
    """Task index, which will not change across job restarts."""
    task_uid: builtins.int
    """Task unique id, which may change across job restarts."""
    def __init__(
        self,
        *,
        name: builtins.str | None = ...,
        cell: builtins.str | None = ...,
        user: builtins.str | None = ...,
        uid: builtins.int | None = ...,
        task_id: builtins.int | None = ...,
        task_uid: builtins.int | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing.Literal[
            "_cell",
            b"_cell",
            "_name",
            b"_name",
            "_task_id",
            b"_task_id",
            "_task_uid",
            b"_task_uid",
            "_uid",
            b"_uid",
            "_user",
            b"_user",
            "cell",
            b"cell",
            "name",
            b"name",
            "task_id",
            b"task_id",
            "task_uid",
            b"task_uid",
            "uid",
            b"uid",
            "user",
            b"user",
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "_cell",
            b"_cell",
            "_name",
            b"_name",
            "_task_id",
            b"_task_id",
            "_task_uid",
            b"_task_uid",
            "_uid",
            b"_uid",
            "_user",
            b"_user",
            "cell",
            b"cell",
            "name",
            b"name",
            "task_id",
            b"task_id",
            "task_uid",
            b"task_uid",
            "uid",
            b"uid",
            "user",
            b"user",
        ],
    ) -> None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal["_cell", b"_cell"]) -> typing.Literal["cell"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal["_name", b"_name"]) -> typing.Literal["name"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal["_task_id", b"_task_id"]) -> typing.Literal["task_id"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal["_task_uid", b"_task_uid"]) -> typing.Literal["task_uid"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal["_uid", b"_uid"]) -> typing.Literal["uid"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal["_user", b"_user"]) -> typing.Literal["user"] | None: ...

global___JobInfo = JobInfo

@typing.final
class CompilationLogEntry(google.protobuf.message.Message):
    """Defines XLA compilation metrics."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _CompilationStage:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _CompilationStageEnumTypeWrapper(
        google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[CompilationLogEntry._CompilationStage.ValueType],
        builtins.type,
    ):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        UNSPECIFIED: CompilationLogEntry._CompilationStage.ValueType  # 0
        END_TO_END: CompilationLogEntry._CompilationStage.ValueType  # 1
        HLO_PASSES: CompilationLogEntry._CompilationStage.ValueType  # 2
        CODE_GENERATION: CompilationLogEntry._CompilationStage.ValueType  # 3
        BACKEND_PASSES: CompilationLogEntry._CompilationStage.ValueType  # 4

    class CompilationStage(_CompilationStage, metaclass=_CompilationStageEnumTypeWrapper):
        """Defines compilation stages for which metrics are collected."""

    UNSPECIFIED: CompilationLogEntry.CompilationStage.ValueType  # 0
    END_TO_END: CompilationLogEntry.CompilationStage.ValueType  # 1
    HLO_PASSES: CompilationLogEntry.CompilationStage.ValueType  # 2
    CODE_GENERATION: CompilationLogEntry.CompilationStage.ValueType  # 3
    BACKEND_PASSES: CompilationLogEntry.CompilationStage.ValueType  # 4

    TIMESTAMP_FIELD_NUMBER: builtins.int
    STAGE_FIELD_NUMBER: builtins.int
    DURATION_FIELD_NUMBER: builtins.int
    TASK_INDEX_FIELD_NUMBER: builtins.int
    PASS_METRICS_FIELD_NUMBER: builtins.int
    MODULE_IDS_FIELD_NUMBER: builtins.int
    JOB_INFO_FIELD_NUMBER: builtins.int
    stage: global___CompilationLogEntry.CompilationStage.ValueType
    """Compilation stage recorded by this log entry."""
    task_index: builtins.int
    """Task index from which this log entry was recorded or
    -1 if the task index could not be fetched. In the case task_index is not
    equal to -1, it is guaranteed to match the task_id in job_info.
    """
    @property
    def timestamp(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """Time when the event captured by this log entry occurred."""

    @property
    def duration(self) -> google.protobuf.duration_pb2.Duration:
        """Duration of the given compilation stage."""

    @property
    def pass_metrics(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___PassMetrics]:
        """Pass specific metrics."""

    @property
    def module_ids(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]:
        """IDs of modules on which the compilation stage was run."""

    @property
    def job_info(self) -> global___JobInfo:
        """Job information."""

    def __init__(
        self,
        *,
        timestamp: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        stage: global___CompilationLogEntry.CompilationStage.ValueType | None = ...,
        duration: google.protobuf.duration_pb2.Duration | None = ...,
        task_index: builtins.int | None = ...,
        pass_metrics: collections.abc.Iterable[global___PassMetrics] | None = ...,
        module_ids: collections.abc.Iterable[builtins.int] | None = ...,
        job_info: global___JobInfo | None = ...,
    ) -> None: ...
    def HasField(
        self, field_name: typing.Literal["duration", b"duration", "job_info", b"job_info", "timestamp", b"timestamp"]
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "duration",
            b"duration",
            "job_info",
            b"job_info",
            "module_ids",
            b"module_ids",
            "pass_metrics",
            b"pass_metrics",
            "stage",
            b"stage",
            "task_index",
            b"task_index",
            "timestamp",
            b"timestamp",
        ],
    ) -> None: ...

global___CompilationLogEntry = CompilationLogEntry



from collections.abc import Collection
from typing import Any

def numsplit(text: str) -> list[str | int]: ...

class ListPortInfo:
    device: str
    name: str
    description: str
    hwid: str
    # USB specific data: the vid and pid attributes below are specific to USB devices only and
    # should be marked as Optional. Since the majority of the serial devices nowadays are USB
    # devices, typing them as Optional will be unnecessarily annoying. We type them with as
    # `int | Any` so that obvious typing errors like ListPortInfo.pid + "str" are flagged.
    # As desired, this will cause a false negative if the value is ever None, but may also cause
    # other false negatives from the Any proliferating. The other USB attributes are correctly
    # typed as Optional because they may be `None` even for USB devices
    # Original discussion at https://github.com/python/typeshed/pull/9347#issuecomment-1358245865.
    vid: int | Any
    pid: int | Any
    serial_number: str | None
    location: str | None
    manufacturer: str | None
    product: str | None
    interface: str | None
    def __init__(self, device: str, skip_link_detection: bool = False) -> None: ...
    def usb_description(self) -> str: ...
    def usb_info(self) -> str: ...
    def apply_usb_info(self) -> None: ...
    def __lt__(self, other: ListPortInfo) -> bool: ...
    def __getitem__(self, index: int) -> str: ...

def list_links(devices: Collection[str]) -> list[str]: ...



from _typeshed import Incomplete
from decimal import Decimal

class MonetaryAmountInput:
    def __init__(self, value: Decimal | None = None, currency_code: str | None = None) -> None: ...
    def to_graphql_variables(self) -> dict[str, Incomplete]: ...
